/*
 * Copyright (C) 2009 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.example.android.BluetoothChat;

//import java.io.UnsupportedEncodingException;

import java.io.UnsupportedEncodingException;
import java.text.SimpleDateFormat;
import java.util.Date;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Intent;
import android.graphics.PorterDuff;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.view.View.OnClickListener;
import android.view.inputmethod.EditorInfo;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemSelectedListener;
import android.view.inputmethod.InputConnection;
import android.view.inputmethod.BaseInputConnection;
import android.inputmethodservice.InputMethodService;
import android.inputmethodservice.Keyboard;
import android.inputmethodservice.KeyboardView;
import android.view.KeyCharacterMap;
import android.view.inputmethod.CompletionInfo;
import android.view.inputmethod.InputMethodManager;

import com.planet.rmu.rfid.R;

/**
 * This is the main Activity that displays the current chat session.
 */
public class BluetoothChat extends Activity {
	// Debugging
	private static final String TAG = "BluetoothChat";
	private static final boolean D = true;

	int i = 0;
	int izt = 0;
	String zzc; 
	String sendMsg;
	
	String flag = "00";
	private static final String[] m={"Reserved","UII","TID","User"};
	private Spinner spinner;  
	private ArrayAdapter<String> adapter; 
	
	// Message types sent from the BluetoothChatService Handler
	public static final int MESSAGE_STATE_CHANGE = 1;
	public static final int MESSAGE_READ = 2; //
	public static final int MESSAGE_WRITE = 3;
	public static final int MESSAGE_DEVICE_NAME = 4;
	public static final int MESSAGE_TOAST = 5;

	// Key names received from the BluetoothChatService Handler
	public static final String DEVICE_NAME = "device_name";
	public static final String TOAST = "toast";

	// Intent request codes
	private static final int REQUEST_CONNECT_DEVICE = 1;
	private static final int REQUEST_ENABLE_BT = 2;

	// Layout Views
	private TextView mTitle;
	private ListView mConversationView;
	private EditText mOutEditText;
	private Button mSendButton;
	private Button mClearButton;
	private EditText meditTextcsk;
	private EditText meditTextslk;
	private EditText meditTextwd;
	private Button mReadButton;
	private Button mWriteButton;

	// Name of the connected device
	private String mConnectedDeviceName = null;
	// Array adapter for the conversation thread
	private ArrayAdapter<String> mConversationArrayAdapter;
	// String buffer for outgoing messages
	private StringBuffer mOutStringBuffer;
	private BluetoothAdapter mBluetoothAdapter = null;
	// Member object for the chat services
	private BluetoothChatService mChatService = null;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		zzc = "";
		if (D)
			Log.e(TAG, "+++ ON CREATE +++");

		// Set up the window layout
		requestWindowFeature(Window.FEATURE_CUSTOM_TITLE);
		setContentView(R.layout.main);
		getWindow().setFeatureInt(Window.FEATURE_CUSTOM_TITLE,
				R.layout.custom_title);

		// Set up the custom title
		mTitle = (TextView) findViewById(R.id.title_left_text);
		mTitle.setText(R.string.app_name);
		mTitle = (TextView) findViewById(R.id.title_right_text);

		// Get local Bluetooth adapter
		mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

		// If the adapter is null, then Bluetooth is not supported
		if (mBluetoothAdapter == null) {
			Toast.makeText(this, "Bluetooth is not available",
					Toast.LENGTH_LONG).show();
			finish();
			return;
		}
		
		meditTextcsk = (EditText) findViewById(R.id.editTextcsk);
		meditTextcsk.setText("01");  
		
		meditTextslk = (EditText) findViewById(R.id.editTextslk);
		meditTextslk.setText("01");  
		
		meditTextwd = (EditText) findViewById(R.id.editTextwd);
		meditTextwd.setText("0000");  
		
		spinner = (Spinner) findViewById(R.id.spinner1);  
		adapter = new ArrayAdapter<String>(this,android.R.layout.simple_spinner_item,m);  
		adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);  
		spinner.setAdapter(adapter);  
		spinner.setOnItemSelectedListener(new SpinnerSelectedListener());  
		spinner.setVisibility(View.VISIBLE);  
		
		mReadButton = (Button) findViewById(R.id.button_read);
		mReadButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				String scsk = meditTextcsk.getText().toString().trim();
				String sslk = meditTextslk.getText().toString().trim();
				if(scsk.length()!=2 | sslk.length()!=2 ){
					mConversationArrayAdapter.add("Parameter error, please enter again!");
					return;
				}
				String sml = "2000000000" + flag + scsk + sslk;
				String slen = Integer.toHexString(sml.length()/2 + 1);
				if (slen.length() == 1)
				{
					slen = "0" + slen;
				}
				sml = "AA" + slen + sml + "55";
				Log.i(TAG, " " + sml);
				izt = 1;
				try {
					sendMessage(sml);
				} catch (UnsupportedEncodingException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		});
		
		mWriteButton = (Button) findViewById(R.id.button_write);
		mWriteButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				String scsk = meditTextcsk.getText().toString().trim();
				String sslk = meditTextslk.getText().toString().trim();
				String sdata = meditTextwd.getText().toString().trim();
				if(scsk.length()!=2 | sslk.length()!=2 | sdata.length()!=4 ){
					mConversationArrayAdapter.add("Parameter error, please enter again!");
					return;
				}
				if(!sslk.equals("01")){
					mConversationArrayAdapter.add("Every time I write only a block!");
					return;
				}
				String sml = "2100000000" + flag + scsk + sslk + sdata;
				
				String slen = Integer.toHexString(sml.length()/2 + 1);
				if (slen.length() == 1)
				{
					slen = "0" + slen;
				}
				sml = "AA" + slen + sml + "55";
				Log.i(TAG, " " + sml);
				izt = 2;
				try {
					sendMessage(sml);
				} catch (UnsupportedEncodingException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}		
		});
		
		
	}

	class SpinnerSelectedListener implements OnItemSelectedListener{  
	       public void onItemSelected(AdapterView<?> arg0, View arg1, int arg2,  
	               long arg3) {  
	    	  flag = Integer.toHexString(arg2);
	    	  if(flag.length()==1)
	    	  {
	    		  flag = "0" + flag;
	    	  }

	       }  
	 
	       public void onNothingSelected(AdapterView<?> arg0) {  
	        }  
	    }  
	
	@Override
	public void onStart() {
		super.onStart();
		if (D)
			Log.e(TAG, "++ ON START ++");

		// If BT is not on, request that it be enabled.
		// setupChat() will then be called during onActivityResult
		if (!mBluetoothAdapter.isEnabled()) {
			Intent enableIntent = new Intent(
					BluetoothAdapter.ACTION_REQUEST_ENABLE);
			startActivityForResult(enableIntent, REQUEST_ENABLE_BT);
			// Otherwise, setup the chat session
		} else {
			if (mChatService == null)
				setupChat();
		}
	}

	@Override
	public synchronized void onResume() {
		super.onResume();
		if (D)
			Log.e(TAG, "+ ON RESUME +");

		// Performing this check in onResume() covers the case in which BT was
		// not enabled during onStart(), so we were paused to enable it...
		// onResume() will be called when ACTION_REQUEST_ENABLE activity
		// returns.
		if (mChatService != null) {
			// Only if the state is STATE_NONE, do we know that we haven't
			// started already
			if (mChatService.getState() == BluetoothChatService.STATE_NONE) {
				// Start the Bluetooth chat services
				mChatService.start();
			}
		}
	}

	private void setupChat() {
		Log.d(TAG, "setupChat()");

		mConversationArrayAdapter = new ArrayAdapter<String>(this,
				R.layout.message);
		mConversationView = (ListView) findViewById(R.id.in);
		mConversationView.setAdapter(mConversationArrayAdapter);

		mClearButton = (Button) findViewById(R.id.button_clear);
		mClearButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				
				mConversationArrayAdapter.clear();
				
			}
		});
		mChatService = new BluetoothChatService(this, mHandler);

		mOutStringBuffer = new StringBuffer("");
	}

	@Override
	public synchronized void onPause() {
		super.onPause();
		if (D)
			Log.e(TAG, "- ON PAUSE -");
	}

	@Override
	public void onStop() {
		super.onStop();
		if (D)
			Log.e(TAG, "-- ON STOP --");
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		if (mChatService != null)
			mChatService.stop();
		if (D)
			Log.e(TAG, "--- ON DESTROY ---");
	}

	private void ensureDiscoverable() {
		if (D)
			Log.d(TAG, "ensure discoverable");
		if (mBluetoothAdapter.getScanMode() != BluetoothAdapter.SCAN_MODE_CONNECTABLE_DISCOVERABLE) {
			Intent discoverableIntent = new Intent(
					BluetoothAdapter.ACTION_REQUEST_DISCOVERABLE);
			discoverableIntent.putExtra(
					BluetoothAdapter.EXTRA_DISCOVERABLE_DURATION, 300);
			startActivity(discoverableIntent);
		}
	}

	/**
	 * Sends a message
	 * 
	 * @param message
	 *            A string of text to send.
	 * @throws UnsupportedEncodingException
	 */
	private void sendMessage(String message)
			throws UnsupportedEncodingException {
		if (mChatService.getState() != BluetoothChatService.STATE_CONNECTED) {
			Toast.makeText(this, R.string.not_connected, Toast.LENGTH_SHORT)
					.show();
			return;
		}

		if (message.length() > 0) {
			byte[] send = CHexConver.hexStr2Bytes(message.toUpperCase());
			mChatService.write(send);
			sendMsg = message;
		}
	}

	private TextView.OnEditorActionListener mWriteListener = new TextView.OnEditorActionListener() {
		public boolean onEditorAction(TextView view, int actionId,
				KeyEvent event) {
			if (actionId == EditorInfo.IME_NULL
					&& event.getAction() == KeyEvent.ACTION_UP) {
				String message = view.getText().toString();
				try {
					sendMessage(message);
				} catch (UnsupportedEncodingException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			if (D)
				Log.i(TAG, "END onEditorAction");
			return true;
		}
	};

	private final Handler mHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			switch (msg.what) {
			case MESSAGE_STATE_CHANGE:
				if (D)
					Log.i(TAG, "MESSAGE_STATE_CHANGE: " + msg.arg1);
				switch (msg.arg1) {
				case BluetoothChatService.STATE_CONNECTED:
					mTitle.setText(R.string.title_connected_to);
					mTitle.append(mConnectedDeviceName);
					mConversationArrayAdapter.clear();
					break;
				case BluetoothChatService.STATE_CONNECTING:
					mTitle.setText(R.string.title_connecting);
					break;
				case BluetoothChatService.STATE_LISTEN:
				case BluetoothChatService.STATE_NONE:
					mTitle.setText(R.string.title_not_connected);
					break;
				}
				break;
			case MESSAGE_WRITE:
				//byte[] writeBuf = (byte[]) msg.obj;
				// construct a string from the buffer
				//String writeMessage = new String(writeBuf);
				//mConversationArrayAdapter.add("Me:  " + sendMsg);
				break;
			case MESSAGE_READ:
				String s3 = String.valueOf(CHexConver.byte2HexStr(
						(byte[]) msg.obj, msg.arg1));
				String readMessage = (new StringBuilder(s3)).append(" ")
						.toString();
				
				//AA 11 10 00 34 00 E3 25 77 DD 8F F2 DC 7B 00 00 09 FD 55
				zzc += readMessage;
				System.out.println(zzc);
				if (zzc.substring(0,2).equals("AA") && zzc.substring(zzc.length() - 3,
						zzc.length() - 1).equals("55")) {
					Date   curDate   =   new   Date(System.currentTimeMillis());
					SimpleDateFormat   formatter   =   new   SimpleDateFormat   ("yyyy-MM-dd HH:mm:ss");  
				    // Date   curDate   =   new   Date(System.currentTimeMillis());//
				    String   str   =   formatter.format(curDate);
				    String s1 = "";
				    String sdata = "";
				    int len = 0;
				    int l = 0;
				    String sslk = meditTextslk.getText().toString().trim();
				    while(zzc.length()>4){
				    	len = (Integer. parseInt(zzc.substring(3,5),16) + 2)*3;
				    	s1 = zzc.substring(6,8);
				    	if(s1.equals("12") && len == 15){
				    		mConversationArrayAdapter.add(mConnectedDeviceName + ": Stop Scanning... 	" + str);
				    	}
				    	if((s1.equals("10") || s1.equals("11")) && len == 15){
				    		mConversationArrayAdapter.add(mConnectedDeviceName + ": Start Scanning... 	" + str);
				    	}
				    	if((s1.equals("10") || s1.equals("11")) && len > 15){
				    		String sid = zzc.substring(18, len - 3);
				    		mConversationArrayAdapter.add(mConnectedDeviceName + ":" + sid +"	" + str);
				    	}
				    	if(s1.equals("20"))
				    	{
				    		if(zzc.subSequence(9, 11).equals("00"))
				    		{
				    			mConversationArrayAdapter.add(mConnectedDeviceName + ": Read data successfully... 	" + str);
				    			l = Integer. parseInt(sslk,16) * 6;
				    			sdata = zzc.substring(18 + l, len - 3);
				    			mConversationArrayAdapter.add(mConnectedDeviceName + ":ID:" + sdata);
				    			sdata = zzc.substring(12, 12 + l);
				    			mConversationArrayAdapter.add(mConnectedDeviceName + ":Datas:" + sdata);
				    		}
				    		else
				    		{
				    			mConversationArrayAdapter.add(mConnectedDeviceName + ": Failed to read data... 	" + str);
				    		}
				    	}
				    	if(s1.equals("21"))
				    	{
				    		if(zzc.subSequence(9, 11).equals("00"))
				    		{
				    			mConversationArrayAdapter.add(mConnectedDeviceName + ": Write data successfully... 	" + str);
				    			
				    			sdata = zzc.substring(18, len - 3);
				    			mConversationArrayAdapter.add(mConnectedDeviceName + ":ID:" + sdata);				    			
				    		}
				    		else
				    		{
				    			mConversationArrayAdapter.add(mConnectedDeviceName + ": Failed to write data... 	" + str);
				    		}
				    	}
				    	zzc = zzc.substring(len);
				    }
					zzc = "";					
				}
				i++;
				break;
			case MESSAGE_DEVICE_NAME:
				// save the connected device's name
				mConnectedDeviceName = msg.getData().getString(DEVICE_NAME);
				Toast.makeText(getApplicationContext(),
						"Connected to " + mConnectedDeviceName,
						Toast.LENGTH_SHORT).show();
				break;
			case MESSAGE_TOAST:
				Toast.makeText(getApplicationContext(),
						msg.getData().getString(TOAST), Toast.LENGTH_SHORT)
						.show();
				break;
			}
		}
	};
	
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		if (D)
			Log.d(TAG, "onActivityResult " + resultCode);
		switch (requestCode) {
		case REQUEST_CONNECT_DEVICE:
			// When DeviceListActivity returns with a device to connect
			if (resultCode == Activity.RESULT_OK) {
				// Get the device MAC address
				String address = data.getExtras().getString(
						Bluetooth_textActivity.EXTRA_DEVICE_ADDRESS);
				BluetoothDevice device = mBluetoothAdapter
						.getRemoteDevice(address);
				mChatService.connect(device);
			}
			break;
		case REQUEST_ENABLE_BT:
			if (resultCode == Activity.RESULT_OK) {
				setupChat();
			} else {
				Log.d(TAG, "BT not enabled");
				Toast.makeText(this, R.string.bt_not_enabled_leaving,
						Toast.LENGTH_SHORT).show();
				finish();
			}
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.option_menu, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.scan:
			// Launch the DeviceListActivity to see devices and do scan
			Intent serverIntent = new Intent(this, Bluetooth_textActivity.class);
			startActivityForResult(serverIntent, REQUEST_CONNECT_DEVICE);
			return true;
		case R.id.discoverable:
			// Ensure this device is discoverable by others
			ensureDiscoverable();
			return true;
		}
		return false;
	}

	/**
	 *
	 */
	private String calcCheckSum(String msg) {
		String resStr="";
		int xor = 0;
		int j = 0;
		String str = "";
		String shex = "";
		shex = msg.toString().trim().replace(" ", "").toUpperCase();
		int len = shex.length();
		str =  shex.substring(0,2);
		xor =  Integer. parseInt(str,16);
		for(int i = 2 ;i < len - 1;i += 2)
		{
		  str = shex.substring(i,i+2);
		  j = Integer. parseInt(str,16);
		  xor = xor ^ j;
		}
		resStr = Integer.toHexString(xor);
		return resStr;
	}


	private String byte2hex(byte b) {
		StringBuffer buf = new StringBuffer();
		char[] hexChars = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
				'A', 'B', 'C', 'D', 'E', 'F' };
		int high = ((b & 0xf0) >> 4);
		int low = (b & 0x0f);
		buf.append(hexChars[high]);
		buf.append(hexChars[low]);

		return buf.toString();
	}

}