package com.planet.rmu.rfid.ui.activity;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.View;
import android.widget.Button;

import com.common.app.base.BaseDataBindingActivity;
import com.common.app.databinding.ViewToolbarBinding;
import com.common.app.ui.bar.ToolBarTop;
import com.common.framework.util.AspLog;
import com.common.framework.view.recyclerview.OnRecyclerItemClickListener;
import com.planet.rmu.rfid.R;
import com.planet.rmu.rfid.databinding.ActivityBluetoothListBinding;
import com.planet.rmu.rfid.ui.adapter.RAdapterSimpleListItem;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * Created by hongfei02.chen on 2019-10-27.
 */
public class ConnectBlueToothActivity extends BaseDataBindingActivity<ActivityBluetoothListBinding>
        implements View.OnClickListener {
    private ToolBarTop mToolbarTop;

    private Button mBtnScan;

    protected List<String> mDataListPaired = new ArrayList<>();
    protected List<String> mDataListNew = new ArrayList<>();
    protected String mTitle;

    private RecyclerView.Adapter mAdapterPaired;
    private RecyclerView.Adapter mAdapterNew;
    private BluetoothAdapter mBtAdapter;

    public static String EXTRA_DEVICE_ADDRESS = "device_address";

    /**
     * 界面参数
     */
    protected RecyclerView mRecyclerViewPaired;
    protected RecyclerView mRecyclerViewNew;
    protected RecyclerView.Adapter mAdapter;


    @Override
    protected int getContentViewId() {
        return R.layout.activity_bluetooth_list;
    }

    @Override
    protected int getTopbarViewId() {
        return R.layout.view_toolbar;
    }

    @Override
    protected void initParams(Bundle savedInstanceState) {
        mActivity = this;
        mBtAdapter = BluetoothAdapter.getDefaultAdapter();

    }

    @Override
    protected void initView() {
        mToolbarTop = ((ViewToolbarBinding) getTopbarViewBinding()).toolbarTop;
        mToolbarTop.setLeftBack(this);
        mToolbarTop.setTitle("connect devices");


        mBtnScan = getContentViewBinding().btnScan;
        mBtnScan.setOnClickListener(this);

        mRecyclerViewPaired = getContentViewBinding().pairedDevices;
        mRecyclerViewNew = getContentViewBinding().newDevices;

        setLayoutManager();
        setAdapter();

        setDecoration();
        mRecyclerViewPaired.addOnItemTouchListener(new OnRecyclerItemClickListener(mRecyclerViewPaired) {
            @Override
            public void onItemClick(RecyclerView.ViewHolder vh, int position) {
                itemClick(vh, position, mDataListPaired.get(position));
            }

            @Override
            public void onItemLongClick(RecyclerView.ViewHolder vh, int position) {
//                itemLongClick(vh, position);
            }
        });
        mRecyclerViewNew.addOnItemTouchListener(new OnRecyclerItemClickListener(mRecyclerViewNew) {
            @Override
            public void onItemClick(RecyclerView.ViewHolder vh, int position) {
                itemClick(vh, position, mDataListNew.get(position));
            }

            @Override
            public void onItemLongClick(RecyclerView.ViewHolder vh, int position) {
//                itemLongClick(vh, position);
            }
        });

        IntentFilter filter = new IntentFilter(BluetoothDevice.ACTION_FOUND);
        this.registerReceiver(mReceiver, filter);

        filter = new IntentFilter(BluetoothAdapter.ACTION_DISCOVERY_FINISHED);
        this.registerReceiver(mReceiver, filter);
    }

    @Override
    protected void loadData() {
        Set<BluetoothDevice> pairedDevices = mBtAdapter.getBondedDevices();

        for (BluetoothDevice device : pairedDevices) {

            mDataListPaired.add(device.getName() + "\n" + device.getAddress());
        }

        mAdapterPaired.notifyDataSetChanged();
    }

    protected void setLayoutManager() {
        LinearLayoutManager layoutManager = new LinearLayoutManager(mActivity);
        layoutManager.setOrientation(LinearLayoutManager.VERTICAL);
        mRecyclerViewPaired.setLayoutManager(layoutManager);

        LinearLayoutManager layoutManager2 = new LinearLayoutManager(mActivity);
        layoutManager2.setOrientation(LinearLayoutManager.VERTICAL);
        mRecyclerViewNew.setLayoutManager(layoutManager2);
    }

    protected void setAdapter() {
        mAdapterPaired = new RAdapterSimpleListItem(this, mDataListPaired, "");
        mAdapterNew = new RAdapterSimpleListItem(this, mDataListNew, "");


        mRecyclerViewPaired.setAdapter(mAdapterPaired);
        mRecyclerViewNew.setAdapter(mAdapterNew);
    }

    protected void setDecoration() {
//        mRecyclerView.addItemDecoration(new RecyclerViewDivider(mActivity));
    }

    private void doDiscovery() {


        if (mBtAdapter.isDiscovering()) {
            mBtAdapter.cancelDiscovery();
        }

        mBtAdapter.startDiscovery();
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btn_scan:
                doDiscovery();
                mBtnScan.setVisibility(View.GONE);
                break;
        }
    }

    protected void itemClick(RecyclerView.ViewHolder vh, int position, String device) {
        mBtAdapter.cancelDiscovery();


        String address = device.substring(device.length() - 17);

        Intent intent = new Intent();
        intent.putExtra(EXTRA_DEVICE_ADDRESS, address);

        setResult(Activity.RESULT_OK, intent);
        finish();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();

        if (mBtAdapter != null) {
            mBtAdapter.cancelDiscovery();
        }

        this.unregisterReceiver(mReceiver);
    }

    private final BroadcastReceiver mReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();

            if (BluetoothDevice.ACTION_FOUND.equals(action)) {
                BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);

                if (device.getBondState() != BluetoothDevice.BOND_BONDED) {

                    mDataListNew.add(device.getName() + "\n" + device.getAddress());
                }
                AspLog.e("jsonjosn", "state: " + device.getBondState()
                        + " " + device.getName() + "\n" + device.getAddress());
            } else if (BluetoothAdapter.ACTION_DISCOVERY_FINISHED.equals(action)) {

                if (mDataListNew.size() == 0) {
                    String noDevices = getResources().getText(R.string.none_found).toString();
                    mDataListNew.add(noDevices);
                }
            }
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    mAdapterNew.notifyDataSetChanged();
                }
            });
        }
    };
}
