package com.common.app.base;

import android.annotation.TargetApi;
import android.content.Intent;
import android.databinding.ViewDataBinding;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.webkit.WebResourceError;
import android.webkit.WebResourceRequest;
import android.webkit.WebView;
import android.webkit.WebViewClient;

import com.common.app.R;
import com.common.app.databinding.ViewWebviewBinding;
import com.common.framework.util.AspLog;
import com.common.framework.util.StringUtil;
import com.common.framework.view.customview.ProgressWebView;


/**
 * Created by chenhongfei on 2016/6/3.
 */
public abstract class BaseAbstractWebViewActivity<VDB extends ViewDataBinding> extends BaseDataBindingActivity<ViewWebviewBinding> {
    private static final String TAG = "BaseAbstractWebViewActivity";

    protected String mTitle;
    protected String mUrl;
    protected boolean mIsLoadingError = false;

    /**
     * 界面参数
     */
    protected ProgressWebView mWvContent;

    @Override
    protected int getContentViewId() {
        return R.layout.view_webview;
    }


    @Override
    protected void initParams(Bundle savedInstanceState) {
        mActivity = this;
        Intent intent = getIntent();
        mTitle = intent.getStringExtra("title");
        mUrl = intent.getStringExtra("url");
    }

    @Override
    protected void initView() {
        mWvContent = getContentViewBinding().wvContent;
        initWebView();

        setViewInfo();
    }


    protected void initWebView() {
        mWvContent.getSettings().setJavaScriptEnabled(true);
        mWvContent.setWebViewClient(
                new WebViewClient() {
                    @Override
                    public boolean shouldOverrideUrlLoading(WebView view, String url) {
                        if (url != null && (url.startsWith("http://") || url.startsWith("https://") || url.startsWith("file://"))) {
                            view.loadUrl(url);
                            return true;
                        } else {
                            view.getContext().startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse(url)));
                            return true;

                        }
                    }

                    @SuppressWarnings("deprecation")
                    @Override
                    public void onReceivedError(WebView view, int errorCode, String description, String failingUrl) {
                        AspLog.v(TAG, String.format("onReceivedError... + errorCode:%d, description:%s, failingUrl:%s ", errorCode, description, failingUrl));
                        super.onReceivedError(view, errorCode, description, failingUrl);
                        mWvContent.setVisibility(View.GONE);

                        showErrorView();
                        mIsLoadingError = true;
                    }

                    @TargetApi(android.os.Build.VERSION_CODES.M)
                    @Override
                    public void onReceivedError(WebView view, WebResourceRequest req, WebResourceError rerr) {
                        // Redirect to deprecated method, so you can use it in all SDK versions
                        onReceivedError(view, rerr.getErrorCode(), rerr.getDescription().toString(), req.getUrl().toString());
                    }

                    @Override
                    public void onPageStarted(WebView view, String url, Bitmap favicon) {
                        AspLog.v(TAG, "onPageStarted:" + url);
                        super.onPageStarted(view, url, favicon);
                        handlePageStarted(url);
                    }

                    @Override
                    public void onPageFinished(WebView view, String url) {
                        AspLog.v(TAG, "onPageFinished...");
                        super.onPageFinished(view, url);
                        handlePageFinished();
                    }
                }
        );

    }

    protected void setViewInfo() {
        if (!TextUtils.isEmpty(mUrl)) {
            mUrl = StringUtil.formatUrl(mUrl);
            mWvContent.loadUrl(mUrl);
        } else {
            showErrorView();
        }
    }

    protected void handlePageFinished() {

    }

    protected void handlePageStarted(String url) {
    }

    @Override
    public void onBackPressed() {
        if (mWvContent.canGoBack()) {
            mWvContent.goBack();
        } else {
            super.onBackPressed();
        }
    }
}
