package com.common.app.base;

import android.app.Application;
import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.support.v4.content.LocalBroadcastManager;
import android.text.TextUtils;

import com.common.framework.network.volley.LruBitmapCache;
import com.common.framework.util.AppUtil;
import com.common.framework.util.AspLog;
import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.toolbox.Volley;

/**
 * Created by alexgu on 2017/2/19.
 */

public abstract class BaseApplication extends Application {
    private static final String TAG = "BaseApplication";
    private static BaseApplication sInstance;
    protected static Context sContext;

    private RequestQueue mRequestQueue;
    public static String CHANNEL_VALUE;
    public static String VERSION_NAME;
    public static String APP_TYPE_VALUE;
    private com.android.volley.toolbox.ImageLoader mImageLoader;
    protected LocalBroadcastManager mLocalBroadcastManager;

    public BaseApplication() {
        super();
        sInstance = this;
    }

    @Override
    public void onCreate() {
        super.onCreate();
        sContext = getApplicationContext();
        CHANNEL_VALUE = getApplicationChannel();
        mLocalBroadcastManager = LocalBroadcastManager.getInstance(sContext);

        initParam();
    }

    public static synchronized BaseApplication getInstance() {
        return sInstance;
    }

    public String getApplicationChannel() {
        return AppUtil.getMetaDataValue(getApplicationContext(), "CHANNEL");
    }

    public String getAppType() {
        return APP_TYPE_VALUE;
    }

    public String getAppVersionName() {
        return AppUtil.getVersionName(getApplicationContext());
    }

    public LocalBroadcastManager getLocalBroadcastManager() {
        return mLocalBroadcastManager;
    }

    public RequestQueue getRequestQueue() {
        if (mRequestQueue == null) {
            mRequestQueue = Volley.newRequestQueue(getApplicationContext());
        }

        return mRequestQueue;
    }

    public void addToRequestQueue(Request req, String tag) {
        req.setTag(TextUtils.isEmpty(tag) ? TAG : tag);
//        VolleyLog.d("Adding request to queue: %s", req.getUrl());
        getRequestQueue().add(req);
    }

    public void addToRequestQueue(Request req) {
        // set the default tag if tag is empty
        req.setTag(TAG);

        getRequestQueue().add(req);
    }

    public void cancelPendingRequests(Object tag) {
        if (mRequestQueue != null) {
            mRequestQueue.cancelAll(tag);
        }
    }

    public com.android.volley.toolbox.ImageLoader getImageLoader() {
        getRequestQueue();
        if (mImageLoader == null) {
            mImageLoader = new com.android.volley.toolbox.ImageLoader(this.mRequestQueue, new LruBitmapCache());
        }
        return this.mImageLoader;
    }

    protected abstract void initParam();
}
