package com.common.app.base;

import android.databinding.ViewDataBinding;
import android.os.Bundle;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;

import com.common.app.R;
import com.common.framework.view.recyclerview.OnRecyclerItemClickListener;
import com.common.framework.view.recyclerview.RecyclerViewDivider;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by chenhongfei on 2017/4/26.
 */

public abstract class BaseBindingRecyclerViewActivity<D, VDB extends ViewDataBinding> extends BaseDataBindingActivity<VDB> {
    private static final String TAG = "BaseBindingRecyclerViewFragment";

    protected List<D> mDataList = new ArrayList<>();
    protected String mTitle;
    protected static int sPageSize = 20;
    protected int mPage = 1;

    /**
     * 界面参数
     */
    protected RecyclerView mRecyclerView;
    protected RecyclerView.Adapter mAdapter;


    @Override
    protected void initParams(Bundle savedInstanceState) {
        mActivity = this;
        mTitle = getIntent().getStringExtra("title");
    }

    @Override
    protected int getContentViewId() {
        return R.layout.common_recyclerview;
    }

    @Override
    protected void initView() {
        mRecyclerView = getRecyclerView();

        setLayoutManager();
        mAdapter = setAdapter(mDataList);
        mRecyclerView.setAdapter(mAdapter);
        setDecoration();
        mRecyclerView.addOnItemTouchListener(new OnRecyclerItemClickListener(mRecyclerView) {
            @Override
            public void onItemClick(RecyclerView.ViewHolder vh, int position) {
                itemClick(vh, position);
            }

            @Override
            public void onItemLongClick(RecyclerView.ViewHolder vh, int position) {
                itemLongClick(vh, position);
            }
        });
        mAdapter.registerAdapterDataObserver(mDataObserver);
        mDataObserver.onChanged();
    }

    @Override
    protected void loadData() {

    }

    protected void setLayoutManager() {
        LinearLayoutManager layoutManager = new LinearLayoutManager(mActivity);
        layoutManager.setOrientation(LinearLayoutManager.VERTICAL);
        mRecyclerView.setLayoutManager(layoutManager);
    }

    protected void setDecoration() {
        mRecyclerView.addItemDecoration(new RecyclerViewDivider(mActivity));
    }

    @Override
    protected void onDestroy() {
        mAdapter.unregisterAdapterDataObserver(mDataObserver);
        super.onDestroy();
    }

    protected abstract RecyclerView getRecyclerView();

    protected abstract RecyclerView.Adapter setAdapter(List<D> dataList);

    public abstract void itemClick(RecyclerView.ViewHolder vh, int position);

    public abstract void itemLongClick(RecyclerView.ViewHolder vh, int position);

    RecyclerView.AdapterDataObserver mDataObserver = new RecyclerView.AdapterDataObserver() {
        @Override
        public void onChanged() {
            super.onChanged();
            if (mAdapter.getItemCount() == 0) {
                showEmptyView();
            } else {
                showContentView();
            }
            onDataChanged();
        }
    };

    protected void onDataChanged () {

    }
}
