package com.common.app.base;

import android.databinding.ViewDataBinding;
import android.os.Bundle;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.View;

import com.common.framework.view.recyclerview.OnRecyclerItemClickListener;
import com.common.framework.view.recyclerview.RecyclerViewDivider;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by chenhongfei on 2017/4/26.
 */

public abstract class BaseBindingRecyclerViewFragment<D, VDB extends ViewDataBinding> extends BaseDataBindingFragment<VDB> {
    private static final String TAG = "BaseBindingRecyclerViewFragment";

    protected List<D> mDataList = new ArrayList<>();
    protected String mTitle;

    /**
     * 界面参数
     */
    protected RecyclerView mRecyclerView;
    protected RecyclerView.Adapter mAdapter;


    @Override
    protected void initParams(Bundle savedInstanceState) {
        mActivity = getActivity();
        if (null != getArguments()) {
            mTitle = getArguments().getString("title");
        }

    }

    @Override
    protected void initView(View view) {
        mRecyclerView = getRecyclerView();

        setLayoutManager();
        mAdapter = setAdapter(mDataList);
        mRecyclerView.setAdapter(mAdapter);
        setDecoration();
        mRecyclerView.addOnItemTouchListener(new OnRecyclerItemClickListener(mRecyclerView) {
            @Override
            public void onItemClick(RecyclerView.ViewHolder vh, int position) {
                itemClick(vh, position);
            }

            @Override
            public void onItemLongClick(RecyclerView.ViewHolder vh, int position) {
                itemLongClick(vh, position);
            }
        });
        mAdapter.registerAdapterDataObserver(mDataObserver);
        mDataObserver.onChanged();
    }

    @Override
    protected void loadData() {

    }

    protected void setLayoutManager() {
        LinearLayoutManager layoutManager = new LinearLayoutManager(mActivity);
        layoutManager.setOrientation(LinearLayoutManager.VERTICAL);
        mRecyclerView.setLayoutManager(layoutManager);
    }

    protected void setDecoration() {
        mRecyclerView.addItemDecoration(new RecyclerViewDivider(mActivity));
    }

    @Override
    public void onDestroyView() {
        mAdapter.unregisterAdapterDataObserver(mDataObserver);
        super.onDestroyView();
    }

    protected abstract RecyclerView getRecyclerView();

    protected abstract RecyclerView.Adapter setAdapter(List<D> dataList);

    public abstract void itemClick(RecyclerView.ViewHolder vh, int position);

    public abstract void itemLongClick(RecyclerView.ViewHolder vh, int position);

    RecyclerView.AdapterDataObserver mDataObserver = new RecyclerView.AdapterDataObserver() {
        @Override
        public void onChanged() {
            super.onChanged();
            if (mAdapter.getItemCount() == 0) {
                showEmptyView();
            } else {
                showContentView();
            }
        }
    };
}
