package com.common.app.base;

import android.databinding.ViewDataBinding;
import android.os.Bundle;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;

import com.common.app.R;
import com.common.framework.network.parser.ErrorInfo;
import com.common.framework.network.parser.ModelInfo;
import com.common.framework.network.parser.NetHandler;
import com.common.framework.util.AspLog;
import com.common.framework.util.ToastUtil;
import com.common.framework.view.recyclerview.OnRecyclerItemClickListener;
import com.common.framework.view.recyclerview.RecyclerViewDivider;
import com.common.framework.view.recyclerview.XRecyclerView;
import com.jcodecraeer.xrecyclerview.ProgressStyle;



import java.util.ArrayList;
import java.util.List;

/**
 * Created by chenhongfei on 2017/4/7.
 */

public abstract class BaseBindingXRecyclerViewActivityNew<M extends ModelInfo, D, VDB extends ViewDataBinding> extends BaseDataBindingActivity<VDB> implements XRecyclerView.LoadingListener, IYsPullRecyclerView {
    private static final String TAG = "BaseXRecyclerViewFragment";
    protected static int sPageSize = 20;
    protected int mPage = 1;
    protected int mTotal;
    protected List<D> mDataList = new ArrayList<>();
    protected String mTitle;
    protected boolean mIsPullDown = true;

    /**
     * 界面参数
     */
    protected SwipeRefreshLayout mSwipeRefreshLayout;
    protected XRecyclerView mRecyclerView;
    protected RecyclerView.Adapter mAdapter;

    @Override
    protected int getContentViewId() {
        return R.layout.common_xrecyclerview;
    }

    @Override
    protected void initParams(Bundle savedInstanceState) {
        mActivity = this;
        mTitle = getIntent().getStringExtra("title");
        mEmptyTip = "暂无内容哦～";
        mIsPullDown = true;
        mPage = 1;
    }

    @Override
    protected void initView() {
        mSwipeRefreshLayout = getSwipeRefreshLayout();
        mRecyclerView = getXRecyclerView();

        setLayoutManager();

        mRecyclerView.setRefreshProgressStyle(ProgressStyle.BallSpinFadeLoader);
        mRecyclerView.setLoadingMoreProgressStyle(ProgressStyle.BallRotate);
        mRecyclerView.setEmptyView(getEmptyView());

        mRecyclerView.setLoadingListener(this);
        mAdapter = setAdapter(mDataList);
        mRecyclerView.setAdapter(mAdapter);
        setItemDecoration();
        mRecyclerView.addOnItemTouchListener(new OnRecyclerItemClickListener(mRecyclerView) {
            @Override
            public void onItemClick(RecyclerView.ViewHolder vh, int position) {
                itemClick(vh, position);
            }

            @Override
            public void onItemLongClick(RecyclerView.ViewHolder vh, int position) {

            }
        });
        setRefreshMode();
        /**
         * XRecyclerView 的下拉刷新会跟addOnItemTouchListener冲突
         * 禁止XRecyclerView 的下拉刷新，采用mSwipeRefreshLayout来下拉刷新
         */
        mRecyclerView.setPullRefreshEnabled(false);
        mSwipeRefreshLayout.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                BaseBindingXRecyclerViewActivityNew.this.onRefresh();
            }
        });
        mSwipeRefreshLayout.setColorSchemeResources(
                R.color.bg_color_progress_bar_red,
                R.color.bg_color_progress_bar_green,
                R.color.bg_color_progress_bar_blue,
                R.color.bg_color_progress_bar_yellow
        );

        getRootViewBinding().setEmptyTip(mEmptyTip);
    }

    protected void setLayoutManager() {
        LinearLayoutManager layoutManager = new LinearLayoutManager(this);
        layoutManager.setOrientation(LinearLayoutManager.VERTICAL);
        mRecyclerView.setLayoutManager(layoutManager);
    }

    @Override
    public void onRefresh() {
        mIsPullDown = true;
        mPage = 1;
        loadData();
    }

    @Override
    public void closeLoadingView() {
        super.closeLoadingView();
        mRecyclerView.refreshComplete();
        mSwipeRefreshLayout.setRefreshing(false);
    }

    @Override
    public void onLoadMore() {
        mIsPullDown = false;
        AspLog.v(TAG, "onLoadMore...");
        int size = mDataList.size();
        if (mTotal <= 0 || mTotal <= size) {
            ToastUtil.toast(this, "无更多的数据...");
            mRecyclerView.setLoadingMoreEnabled(false);
            mRecyclerView.refreshComplete();
            return;
        }

        mPage = size / sPageSize + 1;
        if (mPage < 1) {
            mPage = 1;
        }
        loadData();
    }

    protected void setRefreshMode() {
        mRecyclerView.setLoadingMoreEnabled(false);
    }

    protected void setItemDecoration() {
        mRecyclerView.addItemDecoration(new RecyclerViewDivider(this));
    }

    protected abstract SwipeRefreshLayout getSwipeRefreshLayout();

    protected abstract XRecyclerView getXRecyclerView();

    protected abstract RecyclerView.Adapter setAdapter(List<D> dataList);

    protected void itemClick(RecyclerView.ViewHolder vh, int position) {

    }
//    public abstract void itemLongClick(RecyclerView.ViewHolder vh, int position);

    public abstract void addData(M info);

    protected NetHandler<M> netHandler = new NetHandler<M>() {
        @Override
        public void onGetting() {
            showLoadingView(R.string.loading);
        }

        @Override
        public void onError(int resId, ErrorInfo info) {
//            if (info.checkServerError()) {
//                showErrorView();
//            } else {
//                closeLoadingView();
//            }
            closeLoadingView();
            info.processErrorCode(mActivity);
        }

        @Override
        public void onSuccess(int resId, M info) {
            showContentView();
            if (mIsPullDown) {
                mDataList.clear();
                addData(info);
            } else {
                addData(info);
            }
            mAdapter.notifyDataSetChanged();
        }

        @Override
        public void onDataError() {
            closeLoadingView();
        }

        @Override
        public void onBadLine() {
            showErrorView();
        }
    };
}
