package com.common.app.base;

import android.app.Activity;
import android.databinding.DataBindingUtil;
import android.databinding.ViewDataBinding;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;

import com.common.app.R;
import com.common.app.databinding.BaseViewBinding;
import com.common.app.ui.util.LoadingDialogUtil;
import com.common.app.ui.util.ManageActivity;

/**
 * Created by chenhongfei on 2017/4/14.
 */

public abstract class BaseDataBindingActivity<DBinding extends ViewDataBinding> extends BaseActivity implements IBaseViewShow {
    private BaseViewBinding mBaseViewBinding;
    private DBinding mContentViewBinding;
    private ViewDataBinding mTopbarViewBinding;
    protected String mEmptyTip;
    protected boolean mHasTopbar = true;
    protected Activity mActivity;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mActivity = this;
        setContentView();
        initParams(savedInstanceState);
        setEmptyTip();
        initView();
        showContentView();
        loadData();

        ManageActivity.getInstance().addActivity(this);
    }

    public BaseViewBinding getRootViewBinding() {
        return mBaseViewBinding;
    }

    public ViewDataBinding getTopbarViewBinding() {
        return mTopbarViewBinding;
    }

    public DBinding getContentViewBinding() {
        return mContentViewBinding;
    }

    public View getEmptyView() {
        return mBaseViewBinding.llEmptyContent.llEmptyContent;
    }

    protected void setContentView() {
        mBaseViewBinding = DataBindingUtil.setContentView(this, R.layout.base_view);
        mContentViewBinding = DataBindingUtil.inflate(getLayoutInflater(), getContentViewId(), mBaseViewBinding.rlContainer, true);
        if (getTopbarViewId() > 0 && mHasTopbar) {
            mTopbarViewBinding = DataBindingUtil.inflate(getLayoutInflater(), getTopbarViewId(), mBaseViewBinding.flTitle, true);

        }

        mBaseViewBinding.llNetworkError.llNetworkInvalid.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                retryLoadData();
            }
        });
    }


    protected void setEmptyTip() {
        if (TextUtils.isEmpty(mEmptyTip)) {
            mEmptyTip = this.getResources().getString(R.string.no_data);
        }
        mBaseViewBinding.setEmptyTip(mEmptyTip);
    }

    protected abstract int getContentViewId();

    protected abstract int getTopbarViewId();

    protected abstract void initParams(Bundle savedInstanceState);

    protected abstract void initView();

    protected abstract void loadData();

    protected void retryLoadData() {
        loadData();
    }


    public void showLoadingView(int loadingTip) {
        LoadingDialogUtil.showLoadingDialog(this, loadingTip);
    }

    @Override
    public void showLoadingView(String loadingTip) {
        LoadingDialogUtil.showLoadingDialog(this, loadingTip);
    }

    @Override
    public void showErrorView() {
        closeLoadingView();
        mBaseViewBinding.llEmptyContent.llEmptyContent.setVisibility(View.GONE);
        mBaseViewBinding.llNetworkError.llNetworkInvalid.setVisibility(View.VISIBLE);
    }

    @Override
    public void showEmptyView() {
        closeLoadingView();
        mBaseViewBinding.llEmptyContent.llEmptyContent.setVisibility(View.VISIBLE);
        mBaseViewBinding.llNetworkError.llNetworkInvalid.setVisibility(View.GONE);
    }

    @Override
    public void showContentView() {
        closeLoadingView();
        mBaseViewBinding.llEmptyContent.llEmptyContent.setVisibility(View.GONE);
        mBaseViewBinding.llNetworkError.llNetworkInvalid.setVisibility(View.GONE);
        mBaseViewBinding.rlContainer.setVisibility(View.VISIBLE);
    }

    @Override
    public void closeLoadingView() {
        LoadingDialogUtil.closeLoadingDialog();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        ManageActivity.getInstance().removeActivity(this);
    }
}



