package com.common.app.base;

import android.app.Activity;
import android.app.Dialog;
import android.app.DialogFragment;
import android.databinding.DataBindingUtil;
import android.databinding.ViewDataBinding;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.DisplayMetrics;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.common.app.R;
import com.common.app.databinding.BaseViewBinding;
import com.common.app.ui.util.LoadingDialogUtil;


/**
 * Created by chenhongfei on 2017/4/21.
 */

public abstract class BaseDataBindingDialogFragment<VDB extends ViewDataBinding> extends DialogFragment implements IBaseViewShow {
    private BaseViewBinding mBaseViewBinding;
    private VDB mContentViewBinding;
    private ViewDataBinding mTopbarViewBinding;

    protected Activity mActivity;
    protected Dialog mDialog;
    protected boolean mHasTopbar = true;
    protected String mEmptyTip;

    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        return createDialog(savedInstanceState);
    }

    public Dialog createDialog(Bundle savedInstanceState) {
        mActivity = getActivity();

        LayoutInflater inflater = mActivity.getLayoutInflater();
        mBaseViewBinding = DataBindingUtil.inflate(inflater, R.layout.base_view, null, false);
        mContentViewBinding = DataBindingUtil.inflate(inflater, getContentViewId(), mBaseViewBinding.rlContainer, true);
        if (getTopbarViewId() > 0 && mHasTopbar) {
            mTopbarViewBinding = DataBindingUtil.inflate(inflater, getTopbarViewId(), mBaseViewBinding.flTitle, true);
        }
        View view = mBaseViewBinding.getRoot();

        mDialog = new Dialog(mActivity, R.style.AppDialogTheme);

        mDialog.setContentView(view, getDialogLayoutParams());
        mBaseViewBinding.llNetworkError.llNetworkInvalid.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                retryLoadData();
            }
        });
        setEmptyTip();
        initView(view);

        // 设置点击外围解散
        mDialog.setCanceledOnTouchOutside(true);
        mDialog.setCancelable(true);

        mDialog.show();

        return mDialog;
    }

    public VDB getContentViewBinding() {
        return mContentViewBinding;
    }

    public View getRootView() {
        return mContentViewBinding.getRoot();
    }

    public ViewGroup.LayoutParams getDialogLayoutParams() {
        DisplayMetrics metrics = new DisplayMetrics();
        mActivity.getWindowManager().getDefaultDisplay().getMetrics(metrics);
        return new ViewGroup.LayoutParams(metrics.widthPixels, ViewGroup.LayoutParams.WRAP_CONTENT);
    }

    protected abstract int getContentViewId();

    protected abstract void initView(View view);

    protected abstract void loadData();

    protected int getTopbarViewId() {
        return 0;
    }

    protected void retryLoadData() {
        loadData();
    }

    protected void setEmptyTip() {
        if (TextUtils.isEmpty(mEmptyTip)) {
            mEmptyTip = this.getResources().getString(R.string.no_data);
        }
        mBaseViewBinding.setEmptyTip(mEmptyTip);
    }

    public void showLoadingView(int loadingTip) {
        LoadingDialogUtil.showLoadingDialog(mActivity, loadingTip);
    }

    @Override
    public void showLoadingView(String loadingTip) {
        LoadingDialogUtil.showLoadingDialog(mActivity, loadingTip);
    }

    @Override
    public void showErrorView() {

    }

    @Override
    public void showEmptyView() {

    }

    @Override
    public void showContentView() {

    }

    @Override
    public void closeLoadingView() {
        LoadingDialogUtil.closeLoadingDialog();
    }

}
