package com.common.app.base;

import android.content.Context;
import android.databinding.DataBindingUtil;
import android.databinding.ViewDataBinding;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.PopupWindow;

import com.common.app.R;


/**
 * base popup window
 *
 * @param <VDB>
 */
public abstract class BasePopupWindow<VDB extends ViewDataBinding> implements IYsPopupWindow {
    protected Context mContext;
    protected PopupWindow popupWindow;
    protected PopupWindow.OnDismissListener mDismissListener;
    protected VDB vdBinding;

    public BasePopupWindow() {}

    public BasePopupWindow(Context context, PopupWindow.OnDismissListener dismissListener) {
        mContext = context;
        vdBinding = DataBindingUtil.inflate(LayoutInflater.from(context), getContentViewId(), null, false);
        mDismissListener = dismissListener;
        initPopupWindow();
        initView(vdBinding.getRoot());
    }


    protected void initPopupWindow() {
        popupWindow = new PopupWindow(vdBinding.getRoot(), ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT, true);
        popupWindow.setAnimationStyle(R.style.YsPopupWindowAnim);
        popupWindow.setFocusable(true); // 设置PopupWindow可获得焦点
        popupWindow.setTouchable(true); // 设置PopupWindow可触摸
        popupWindow.setOutsideTouchable(true); // 设置非PopupWindow区域可触摸
        popupWindow.setBackgroundDrawable(mContext.getResources().getDrawable(R.color.transparent));
        setWidthAndHeight(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);

        popupWindow.setOnDismissListener(mDismissListener);
    }

    public void show(View view) {
        popupWindow.showAsDropDown(view);
    }

    protected void setWidthAndHeight(int width, int height) {
        popupWindow.setWidth(width);
        popupWindow.setHeight(height);
    }

    public VDB getContentViewBinding() {
        return vdBinding;
    }

    protected abstract int getContentViewId();

    protected abstract void initView(View view);
}
