package com.common.app.data.network;

import android.content.Context;
import android.os.Message;
import android.text.TextUtils;

import com.android.internal.http.multipart.Part;
import com.android.internal.http.multipart.StringPart;
import com.common.app.R;
import com.common.app.base.BaseApplication;
import com.common.app.data.sharedpreferences.SharedPrefsUserInfo;
import com.common.app.global.ParamsKey;
import com.common.framework.network.parser.BaseParser;
import com.common.framework.network.parser.NetHandler;
import com.common.framework.network.volley.AjaxParams;
import com.common.framework.network.volley.VolleyNetwork;
import com.common.framework.util.AppUtil;
import com.common.framework.util.Installation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by chenhongfei on 2015/7/10.
 */
public class ApiRequest {

    private static String mImei = "";

    /**
     * 获取imei号，缓存在程序中
     *
     * @return
     */
    protected static String getDeviceId() {
        if (!TextUtils.isEmpty(mImei)) {
            return mImei;
        }
        Context context = BaseApplication.getInstance().getApplicationContext();
        mImei = AppUtil.getUUID(context);

        if (TextUtils.isEmpty(mImei)) {
            mImei = Installation.getUuid(context);
        }
        if (TextUtils.isEmpty(mImei)) {
            mImei = "Android-0000-0000-0000";
        }

        return mImei;
    }

    /**
     * appId 已改成和业务相关的参数
     *
     * @param map
     * @return
     */
    protected Map<String, Object> getPublicParams(Map<String, Object> map) {
        String imei = getDeviceId();
        map.put(ParamsKey.CLIENT_VERSION, BaseApplication.VERSION_NAME);
        map.put(ParamsKey.CHANNEL, BaseApplication.CHANNEL_VALUE);
        map.put(ParamsKey.APP_TYPE, BaseApplication.APP_TYPE_VALUE);
        map.put("imei", imei);
        try {
            map.put("user_id", SharedPrefsUserInfo.getInstance().getUserId());
            map.put("country", AppUtil.getCountry(BaseApplication.getInstance().getApplicationContext()));
            map.put("language", AppUtil.getLanguage(BaseApplication.getInstance().getApplicationContext()));
        } catch (Exception e) {
            e.printStackTrace();
        }

        return map;
    }

    protected AjaxParams getPublicParams(AjaxParams params) {
        String imei = getDeviceId();
        params.put(ParamsKey.CLIENT_VERSION, BaseApplication.VERSION_NAME);
        params.put(ParamsKey.CHANNEL, BaseApplication.CHANNEL_VALUE);
        params.put("imei", imei);
        return params;
    }

    protected ArrayList<Part> getPublicParams(ArrayList<Part> params) {
        if (params == null) {
            return params;
        }
        try {
            String imei = getDeviceId();
            params.add(new StringPart(ParamsKey.CLIENT_VERSION, BaseApplication.VERSION_NAME));
//            params.add(new StringPart(ParamsKey.CHANNEL, BaseApplication.CHANNEL_VALUE));
//            params.add(new StringPart(ParamsKey.APP_TYPE, ""));
            params.add(new StringPart("imei", imei));
            params.add(new StringPart("user_id", SharedPrefsUserInfo.getInstance().getUserId()));
            params.add(new StringPart("country", AppUtil.getCountry(BaseApplication.getInstance().getApplicationContext())));
            params.add(new StringPart("language", AppUtil.getLanguage(BaseApplication.getInstance().getApplicationContext())));
        } catch (Exception e) {
            e.printStackTrace();
        }


        return params;

    }


    protected void prepare(NetHandler handler) {
        Message msg = new Message();
        msg.what = R.id.doGetting;
        handler.sendMessage(msg);
    }


    protected void request(Context context, String apiName, HashMap<String, Object> map, BaseParser parser) {
        VolleyNetwork.doHttpPost(ApiParams.REQUEST_URL + apiName, "", map, parser);
    }

    protected void uploadFile(String apiName, AjaxParams params, BaseParser parser) {
        VolleyNetwork.uploadFile(ApiParams.REQUEST_URL + apiName, params, parser);
    }
}
