package com.common.app.data.sharedpreferences;

import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.databinding.BaseObservable;
import android.text.TextUtils;

import com.common.app.base.BaseApplication;
import com.common.app.global.SharedPrefsKey;
import com.google.gson.Gson;


/**
 * Created by chenhongfei on 2016/2/26.
 */
public class SharedPrefsUserInfo extends BaseObservable {
    private static final String TAG = "SharedPrefsUserInfo";

    private static final String SHARED_PREFS_FILE = "userInfo";
    private static SharedPrefsUserInfo mInstance;
    private static SharedPreferences mSharedPrefs;
    private static Context mContext;
    private static Gson mGson = new Gson();

    public synchronized static SharedPrefsUserInfo getInstance() {
        if (mInstance == null) {
            mInstance = new SharedPrefsUserInfo(BaseApplication.getInstance().getApplicationContext());
        }

        return mInstance;
    }

    public synchronized static SharedPrefsUserInfo getInstance(Context context) {
        return getInstance();
    }

    private SharedPrefsUserInfo(Context context) {
        mContext = context;
        mSharedPrefs = mContext.getSharedPreferences(SHARED_PREFS_FILE, Activity.MODE_PRIVATE);
    }

    /**
     * 清除数据
     */
    public void clearUserInfo() {
        String account = mSharedPrefs.getString(SharedPrefsKey.KEY_ACCOUNT, null);

        SharedPreferences.Editor editor = mSharedPrefs.edit();
        editor.clear().commit();
        //保留用户名
        editor.putString(SharedPrefsKey.KEY_ACCOUNT, account).commit();
    }


    public String getSessionId() {
        if (mSharedPrefs != null && mContext != null) {
            return mSharedPrefs.getString(SharedPrefsKey.KEY_SESSION_ID, "");
        }

        return "";
    }

    public boolean saveSessionId(String sessionId) {
        if (TextUtils.isEmpty(sessionId)) {
            return false;
        }
        return mSharedPrefs.edit().putString(SharedPrefsKey.KEY_SESSION_ID, sessionId).commit();
    }

    //得到用户uid
    public String getUserId() {
        try {
            if (mSharedPrefs != null && mContext != null) {
                return mSharedPrefs.getString(SharedPrefsKey.KEY_USER_ID, "");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return "";
    }

    //保存用户uid
    public boolean saveUserId(String userId) {
        return mSharedPrefs.edit().putString(SharedPrefsKey.KEY_USER_ID, userId).commit();
    }


    public String getAccount() {
        if (mSharedPrefs != null && mContext != null) {
            return mSharedPrefs.getString(SharedPrefsKey.KEY_ACCOUNT, null);
        }

        return null;
    }

    public boolean saveAccount(String account) {
        return mSharedPrefs.edit().putString(SharedPrefsKey.KEY_ACCOUNT, account).commit();
    }

    public String getRegCode() {
        if (mSharedPrefs != null && mContext != null) {
            return mSharedPrefs.getString(SharedPrefsKey.KEY_REG_CODE, null);
        }

        return null;
    }


    public boolean saveRegCode(String code) {
        return mSharedPrefs.edit().putString(SharedPrefsKey.KEY_REG_CODE, code).commit();
    }
}
