package com.common.app.ui.bar;

import android.app.Activity;
import android.content.Context;
import android.support.annotation.Nullable;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.common.app.R;


/**
 * Created by apple on 2017/7/2.
 */

public class ToolBarTop extends LinearLayout {
    private Context mContext;
    private View mRootView;
    private LinearLayout mLlLeft, mLlRight;
    private ImageView mIvLeft, mIvRight, mIvMiddle;
    private TextView mTvLeft, mTvRight, mTvMiddle;

    public ToolBarTop(Context context) {
        this(context, null);
    }

    public ToolBarTop(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public ToolBarTop(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initParams(context);
        initView();
    }

    private void initParams(Context context) {
        mContext = context;
    }

    private void initView() {
        mRootView = LayoutInflater.from(mContext).inflate(R.layout.toolbar_top, this);
        mLlLeft = (LinearLayout) mRootView.findViewById(R.id.ll_left);
        mIvLeft = (ImageView) mRootView.findViewById(R.id.iv_left);
        mTvLeft = (TextView) mRootView.findViewById(R.id.tv_left);
        mIvMiddle = (ImageView) mRootView.findViewById(R.id.iv_middle);
        mTvMiddle = (TextView) mRootView.findViewById(R.id.tv_middle);
        mLlRight = (LinearLayout) mRootView.findViewById(R.id.ll_right);
        mIvRight = (ImageView) mRootView.findViewById(R.id.iv_right);
        mTvRight = (TextView) mRootView.findViewById(R.id.tv_right);
    }

    /****************************左边**********************/
    public void setLeftBack(final Activity activity) {
        mIvLeft.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                activity.onBackPressed();
            }
        });
    }

    public void setLeftTextView(String leftText, OnClickListener clickListener) {
        mLlLeft.setVisibility(VISIBLE);
        mIvLeft.setVisibility(GONE);
        mTvLeft.setVisibility(VISIBLE);
        mTvLeft.setText(leftText);

        if (null != clickListener) {
            mTvLeft.setOnClickListener(clickListener);
        }
    }

    public void setLeftTextView(int leftTextId, OnClickListener clickListener) {
        mLlLeft.setVisibility(VISIBLE);
        mIvLeft.setVisibility(GONE);
        mTvLeft.setVisibility(VISIBLE);
        mTvLeft.setText(leftTextId);

        if (null != clickListener) {
            mTvLeft.setOnClickListener(clickListener);
        }
    }

    public void setLeftImageView(int imageResId, OnClickListener clickListener) {
        mLlLeft.setVisibility(VISIBLE);
        mIvLeft.setVisibility(VISIBLE);
        mTvLeft.setVisibility(GONE);
        mIvLeft.setImageResource(imageResId);
        mIvLeft.setOnClickListener(clickListener);
    }

    public void setLeftCircleImageView(int imageResId, OnClickListener clickListener) {
        mLlLeft.setVisibility(VISIBLE);
        mIvLeft.setVisibility(GONE);
        mTvLeft.setVisibility(GONE);
        mIvLeft.setImageResource(imageResId);
        mIvLeft.setOnClickListener(clickListener);
    }

    public void hideLeft() {
        mLlLeft.setVisibility(INVISIBLE);
    }

    /****************************title**********************/
    public void setTitle(String title) {
        mTvMiddle.setText(title);
    }

    public void setTitle(int titleResId) {
        mTvMiddle.setText(titleResId);
    }

    /****************************right**********************/
    public void hideRight() {
        mLlRight.setVisibility(INVISIBLE);
    }

    public void setRightText(String rightText, OnClickListener clickListener) {
        mLlRight.setVisibility(VISIBLE);
        mIvRight.setVisibility(GONE);
        mTvRight.setVisibility(VISIBLE);
        mTvRight.setText(rightText);
        if (null != clickListener) {
            mTvRight.setOnClickListener(clickListener);
        }
    }

    public void setRightText(int rightTextId, OnClickListener clickListener) {
        mLlRight.setVisibility(VISIBLE);
        mIvRight.setVisibility(GONE);
        mTvRight.setVisibility(VISIBLE);
        mTvRight.setText(rightTextId);
        if (null != clickListener) {
            mTvRight.setOnClickListener(clickListener);
        }
    }

    public void setRightImageView(int resId, OnClickListener clickListener) {
        mLlRight.setVisibility(VISIBLE);
        mIvRight.setVisibility(VISIBLE);
        mTvRight.setVisibility(GONE);
        mIvRight.setImageResource(resId);
        mIvRight.setOnClickListener(clickListener);
    }
}
