package com.common.app.ui.popupwindow;

import android.content.Context;
import android.databinding.DataBindingUtil;
import android.databinding.ViewDataBinding;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.AnimationUtils;
import android.widget.PopupWindow;

import com.common.app.R;
import com.common.app.base.BasePopupWindow;
import com.common.app.base.IPopupWindowListener;
import com.common.app.ui.adapter.RAdapterFilter;
import com.common.framework.view.recyclerview.OnRecyclerItemClickListener;


/**
 * Created by chenhongfei on 2017/6/15.
 */

public abstract class BasePopupWindowRecyclerView<VDB extends ViewDataBinding> extends BasePopupWindow<VDB> {
    public static final String TAG = "BasePopupWindowRecyclerView";
    protected RAdapterFilter mAdapter;
    protected IPopupWindowListener.RecyclerViewItemListener mPopupWindowCallback;

    public BasePopupWindowRecyclerView( ) {

    }

    public BasePopupWindowRecyclerView(Context context, RAdapterFilter adapter, final IPopupWindowListener.RecyclerViewItemListener popupWindowCallback) {
        mContext = context;
        mAdapter = adapter;
        mPopupWindowCallback = popupWindowCallback;

        vdBinding = DataBindingUtil.inflate(LayoutInflater.from(context), getContentViewId(), null, false);
        mDismissListener = new PopupWindow.OnDismissListener() {
            @Override
            public void onDismiss() {
                if (null != mPopupWindowCallback) {
                    mPopupWindowCallback.onDismiss();
                }
            }
        };
        initPopupWindow();
        initView(vdBinding.getRoot());
    }

    @Override
    protected void initView(View view) {
        RecyclerView recyclerView = (RecyclerView) view.findViewById(R.id.rv_choices);
        setLayoutManager(recyclerView);
        recyclerView.setAdapter(mAdapter);

        recyclerView.addOnItemTouchListener(new OnRecyclerItemClickListener(recyclerView) {
            @Override
            public void onItemClick(RecyclerView.ViewHolder vh, int position) {
                mAdapter.setSelectedPosition(position);
                if (null != mPopupWindowCallback) {
                    mPopupWindowCallback.onItemClick(vh, position);
                }
                if (null != popupWindow) {
                    popupWindow.dismiss();
                }
            }

            @Override
            public void onItemLongClick(RecyclerView.ViewHolder vh, int position) {

            }
        });

        view.setAnimation(AnimationUtils.loadAnimation(mContext, R.anim.alpha_fast));
    }

    protected void setLayoutManager(RecyclerView recyclerView) {
        LinearLayoutManager layoutManager = new LinearLayoutManager(mContext);
        layoutManager.setOrientation(LinearLayoutManager.VERTICAL);
        recyclerView.setLayoutManager(layoutManager);
    }

    @Override
    protected void setWidthAndHeight(int width, int height) {
        super.setWidthAndHeight(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
    }
}
