package com.common.framework.download;

import android.app.DownloadManager;
import android.app.Service;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.database.Cursor;
import android.net.Uri;
import android.os.IBinder;
import android.text.TextUtils;
import android.widget.Toast;

import com.common.app.data.sharedpreferences.SharedPrefsOther;
import com.common.framework.util.AppUtil;
import com.common.framework.util.AspLog;

import java.io.File;

/**
 * 利用DownloadManager 来下载文件
 * <p/>
 * Created by chenhongfei on 2016/1/12.
 */
public class DownloadService extends Service {
    private static final String TAG = "DownloadService";
    private DownloadManager downloadManager;
    private DownloadCompleteReceiver completeReceiver;

    private String title;
    private String downloadUrl;

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    @Override
    public void onCreate() {
        downloadManager = (DownloadManager) getSystemService(DOWNLOAD_SERVICE);
        super.onCreate();
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        if (intent != null) {
            title = intent.getExtras().getString("title");
            downloadUrl = intent.getExtras().getString("downloadUrl");
        }
        String fileName = String.format("acc369_%s.apk", AppUtil.getVersionName(this));
        AspLog.v(TAG, String.format("title:%s, fileName:%s, downloadUrl:%s", title, fileName, downloadUrl));
        if (!TextUtils.isEmpty(title) && !TextUtils.isEmpty(downloadUrl)) {
            long downloadId = SharedPrefsOther.getInstance().getDownloadId();
            AspLog.v(TAG, "old downloadId:" + downloadId);
            if (downloadId != 0) {
                // 如果正在下载安装包，不重复下载
                if (isDownloading(downloadId)) {
                    // do no thing...
//                    Validate.Toast(this, "正在下载...");
                    Toast.makeText(this, "正在下载...", Toast.LENGTH_SHORT).show();
                } else {
                    initDownloadManager(downloadUrl, title, fileName);
                }
            } else {
                initDownloadManager(downloadUrl, title, fileName);
            }
        }
        return super.onStartCommand(intent, flags, startId);
    }

    private void initDownloadManager(String url, String title, String fileName) {
        String filePath = AppUtil.getCacheDirPath(this) + fileName;
        AspLog.v(TAG, "filePath:" + filePath);
        File downloadFile = new File(filePath);
        if (downloadFile.exists()) {
            downloadFile.delete();
        }

        completeReceiver = new DownloadCompleteReceiver();

        DownloadManager.Request downloadRequest = new DownloadManager.Request(Uri.parse(url));
        // 设置下载允许使用的网络类型
        downloadRequest.setAllowedNetworkTypes(DownloadManager.Request.NETWORK_MOBILE | DownloadManager.Request.NETWORK_WIFI);
        downloadRequest.setNotificationVisibility(DownloadManager.Request.VISIBILITY_VISIBLE_NOTIFY_COMPLETED);
        // 显示下载界面
        downloadRequest.setVisibleInDownloadsUi(true);
//        downloadRequest.setDestinationInExternalFilesDir(this, Environment.DIRECTORY_DOWNLOADS, fileName);
        downloadRequest.setDestinationUri(Uri.fromFile(downloadFile));
        downloadRequest.setTitle(title);
        long downloadId = downloadManager.enqueue(downloadRequest);

        SharedPrefsOther.getInstance().saveDownloadId(downloadId);

        registerReceiver(completeReceiver, new IntentFilter(DownloadManager.ACTION_DOWNLOAD_COMPLETE));
    }

    private boolean isDownloading(long downloadId) {
        boolean isDownloading = false;
        DownloadManager.Query downloadQuery = new DownloadManager.Query();
        downloadQuery.setFilterById(downloadId);
        Cursor cursor = downloadManager.query(downloadQuery);
        if (cursor.moveToFirst()) {
            int status = cursor.getInt(cursor.getColumnIndex(DownloadManager.COLUMN_STATUS));
            switch (status) {
                case DownloadManager.STATUS_PAUSED:
                    AspLog.v(TAG, "status: STATUS_PAUSED");
                    break;
                case DownloadManager.STATUS_PENDING:
                    AspLog.v(TAG, "status: STATUS_PENDING");
                    break;
                case DownloadManager.STATUS_RUNNING:
                    AspLog.v(TAG, "status: STATUS_RUNNING");
                    isDownloading = true;
                    break;
                case DownloadManager.STATUS_SUCCESSFUL:
                    AspLog.v(TAG, "status: STATUS_SUCCESSFUL");
                    break;
                case DownloadManager.STATUS_FAILED:
                    AspLog.v(TAG, "status: STATUS_FAILED");
                    break;
            }
        }

        return isDownloading;
    }

    @Override
    public void onDestroy() {
        if (completeReceiver != null) {
            unregisterReceiver(completeReceiver);
        }
        super.onDestroy();
    }

    class DownloadCompleteReceiver extends BroadcastReceiver {

        @Override
        public void onReceive(Context context, Intent intent) {
            if (intent.getAction().equals(DownloadManager.ACTION_DOWNLOAD_COMPLETE)) {
                long downloadId = intent.getLongExtra(DownloadManager.EXTRA_DOWNLOAD_ID, -1);
                AspLog.v(TAG, "onReceive() downloadId:" + downloadId);
                SharedPrefsOther.getInstance().saveDownloadId(downloadId);
                installApk(downloadManager.getUriForDownloadedFile(downloadId));
                DownloadService.this.stopSelf();
            }
        }

        private void installApk(Uri apk) {
            if (apk == null) {
                AspLog.e(TAG, "warn: apk uri is null...");
                return;
            }
            Intent intent = new Intent();

            intent.setAction(Intent.ACTION_VIEW);
            intent.addCategory("android.intent.category.DEFAULT");
            intent.setDataAndType(apk, "application/vnd.android.package-archive");
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);

            AspLog.v(TAG, "install 369 apk.......");
            startActivity(intent);
        }
    }
}



