package com.common.framework.network.volley;

import com.common.framework.util.AspLog;
import com.android.volley.AuthFailureError;
import com.android.volley.NetworkResponse;
import com.android.volley.ParseError;
import com.android.volley.Request;
import com.android.volley.Response;
import com.android.volley.toolbox.HttpHeaderParser;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;

import java.io.UnsupportedEncodingException;
import java.util.Map;

/**
 * Created by guqingbo on 2016/4/13.
 */
public class FileRequest<T> extends Request<T> {
    private final static String TAG = "FileRequest";
    private final Class<T> mClassType;
    private final AjaxParams mParams;
    private final Gson mGson = new Gson();
    private Response.Listener<T> mListener;
    private Map<String, String> mHeaders;

    public FileRequest(String url, Class<T> classType, AjaxParams params,
                       Response.Listener<T> listener, Response.ErrorListener errorListener) {
        this(Method.POST, url, classType, null, params, listener, errorListener);
    }

    public FileRequest(int method, String url, Class<T> classType, Map<String, String> headers, AjaxParams params, Response.Listener<T> listener,
                       Response.ErrorListener errorListener) {
        super(method, url, errorListener);
        mListener = listener;
        mClassType = classType;
        mParams = params;
        mHeaders = headers;
    }

    @Override
    public Map<String, String> getHeaders() throws AuthFailureError {
        return mHeaders != null ? mHeaders : super.getHeaders();
    }

    @Override
    protected void deliverResponse(T response) {
        if (mListener != null) {
            mListener.onResponse(response);
        }
    }

    @Override
    public String getBodyContentType() {
        return mParams.getBodyContentType();
    }

    @Override
    public byte[] getBody() throws AuthFailureError {
        return mParams.getBody();
    }

    @Override
    protected Response<T> parseNetworkResponse(NetworkResponse networkResponse) {
        try {
            String json = new String(networkResponse.data, HttpHeaderParser.parseCharset(networkResponse.headers));
            int maxLogStringLength = 3000;
            if (json.length() <= maxLogStringLength) {
                AspLog.d(TAG, " ===json class name:" + mClassType.getName() + " response: " + json);
            } else {
                int ptime = (json.length() / maxLogStringLength) + 1;
                for (int i = 0; i < ptime; i++) {
                    AspLog.d(TAG, " ===json class name:" + mClassType.getName() + " response: " +
                            json.substring(maxLogStringLength * i, Math.min(maxLogStringLength * (i + 1), json.length())));
                }
            }

            return Response.success(mGson.fromJson(json, mClassType),
                    HttpHeaderParser.parseCacheHeaders(networkResponse));
        } catch (UnsupportedEncodingException e) {
            return Response.error(new ParseError(e));
        } catch (JsonSyntaxException e) {
            return Response.error(new ParseError(e));
        } catch (Exception e) {
            return Response.error(new ParseError(e));
        }
    }

}
