package com.common.framework.network.volley;

import com.common.framework.util.AspLog;
import com.android.volley.AuthFailureError;
import com.android.volley.NetworkResponse;
import com.android.volley.ParseError;
import com.android.volley.Response;
import com.android.volley.toolbox.HttpHeaderParser;
import com.android.volley.toolbox.JsonRequest;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;

import java.io.UnsupportedEncodingException;
import java.util.Map;

/**
 * volley + gson , 此类继承了JsonRequest
 * <p/>
 * Created by chenhongfei on 2015/10/27.
 */
public class GsonObjectRequest<T> extends JsonRequest<T> {
    private static String TAG = "GsonObjectRequest";

    private final Gson mGson = new Gson();
    private final Class<T> mClassType;
    private final Map<String, String> mHeaders;
    private final Response.Listener<T> mListener;

    public GsonObjectRequest(String url, Class<T> classType, String jsonString,
                             Response.Listener<T> listener, Response.ErrorListener errorListener) {
        this(Method.POST, url, classType, null, jsonString, listener, errorListener);
    }

    public GsonObjectRequest(int method, String url, Class<T> classType, Map<String, String> headers, String jsonString,
                             Response.Listener<T> listener, Response.ErrorListener errorListener) {
        super(method, url, (jsonString == null) ? null : jsonString, listener, errorListener);

        mClassType = classType;
        mHeaders = headers;
        mListener = listener;
    }

    @Override
    public Map<String, String> getHeaders() throws AuthFailureError {
        return mHeaders != null ? mHeaders : super.getHeaders();
    }

    @Override
    protected void deliverResponse(T response) {
        mListener.onResponse(response);
    }

    @Override
    protected Response<T> parseNetworkResponse(NetworkResponse networkResponse) {
        try {
            String json = new String(networkResponse.data, HttpHeaderParser.parseCharset(networkResponse.headers));
            int maxLogStringLength = 3000;
            if (json.length() <= maxLogStringLength) {
                AspLog.d(TAG, " ===json class name:" + mClassType.getName() + " response: " + json);
            } else {
                int ptime = (json.length() / maxLogStringLength) + 1;
                for (int i = 0; i < ptime; i++) {
                    AspLog.d(TAG, " ===json class name:" + mClassType.getName() + " response: " +
                            json.substring(maxLogStringLength * i, Math.min(maxLogStringLength * (i + 1), json.length())));
                }
            }
            return Response.success(mGson.fromJson(json, mClassType),
                    HttpHeaderParser.parseCacheHeaders(networkResponse));
        } catch (UnsupportedEncodingException e) {
            return Response.error(new ParseError(e));
        } catch (JsonSyntaxException e) {
            return Response.error(new ParseError(e));
        } catch (Exception e) {
            return Response.error(new ParseError(e));
        }
    }


}