package com.common.framework.network.volley;

import android.util.Log;
import android.widget.Toast;

import com.android.internal.http.multipart.Part;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.common.app.base.BaseApplication;
import com.common.framework.network.parser.BaseParser;
import com.common.framework.network.parser.ModelInfo;
import com.common.framework.network.parser.ResponseListener;
import com.common.framework.util.AspLog;
import com.android.volley.DefaultRetryPolicy;
import com.google.gson.Gson;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * 此类封装了volley框架的网络请求方法
 * <p/>
 * Created by apple on 15/10/24.
 */
public class VolleyNetwork {
    private static final String TAG = "VolleyNetwork";
    public static final String HTTP_VERSIONID = "001";
    public static final String JSON_RPC_VERSION = "2.0";

    // 默认超时时间，毫秒
    private static final int DEFAULT_TIMEOUT = 5000;
    private static final int DEFAULT_FILE_TIMEOUT = 10000;

    /**
     * 使用GsonObjectRequest，继承了JsonRequest
     *
     * @param url
     * @param method
     * @param map
     * @param baseParser
     */
    public static <T extends ModelInfo> void doHttpPost(final String url, String method, final Map<String, Object> map, final BaseParser<T> baseParser) {


        Gson gson = new Gson();
        String postString = gson.toJson(map);
        int maxLogStringLength = 3000;
        if (postString.length() <= maxLogStringLength) {
            AspLog.d(TAG, "json url:" + url + " postData=" + postString);
        } else {
            int ptime = (postString.length() / maxLogStringLength) + 1;
            for (int i = 0; i < ptime; i++) {
                AspLog.d(TAG, "json url:" + url + " postData=" + postString.substring(maxLogStringLength * i, Math.min(maxLogStringLength * (i + 1), postString.length())));
            }
        }

        Class<T> mClass = baseParser.getModelInfoClass();
        ResponseListener<T> responseListener = new ResponseListener<>(baseParser);
        GsonObjectRequest<T> gsonRequest = new GsonObjectRequest<>(url, mClass,
                postString, responseListener, responseListener);

        gsonRequest.setShouldCache(false);
        gsonRequest.setRetryPolicy(
                new DefaultRetryPolicy(DEFAULT_TIMEOUT, DefaultRetryPolicy.DEFAULT_MAX_RETRIES, DefaultRetryPolicy.DEFAULT_BACKOFF_MULT)
        );

        BaseApplication.getInstance().addToRequestQueue(gsonRequest, TAG);
    }

    public static <T extends ModelInfo> void uploadFile(final String url, final AjaxParams params, final BaseParser<T> baseParser) {
        AspLog.d(TAG, "json url:" + url + " postData=" + params.toString());

        Class<T> mClass = baseParser.getModelInfoClass();
        ResponseListener<T> responseListener = new ResponseListener<>(baseParser);
        FileRequest<T> fileRequest = new FileRequest<>(url, mClass,
                params, responseListener, responseListener);
        fileRequest.setShouldCache(false);
        fileRequest.setRetryPolicy(
                new DefaultRetryPolicy(DEFAULT_FILE_TIMEOUT * 3, DefaultRetryPolicy.DEFAULT_MAX_RETRIES, DefaultRetryPolicy.DEFAULT_BACKOFF_MULT)
        );

        BaseApplication.getInstance().addToRequestQueue(fileRequest, TAG);
    }


    public static <T extends ModelInfo> void uploadFile(final String url, final ArrayList<Part> partList, final BaseParser<T> baseParser) {
        try {
            if (partList == null) {
                AspLog.d(TAG, "json url:" + url + " data: null" );
            } else {
                AspLog.d(TAG, "json url:" + url + " data:" + partList.toString());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        Class<T> mClass = baseParser.getModelInfoClass();
        final ResponseListener<T> responseListener = new ResponseListener<>(baseParser);

        MultipartRequest multipartRequest = new MultipartRequest(url, partList.toArray(new Part[partList.size()]), new Response.Listener<String>() {
            @Override
            public void onResponse(String response) {
//               responseListener.onResponse(response);
                AspLog.d(TAG, "jsonjson response:" + response);
                baseParser.parseResponse(response);

            }
        }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
//                //处理失败错误信息
                AspLog.e(TAG, error.getMessage(), error);
                baseParser.parseResponse("");
            }
        });
        multipartRequest.setRetryPolicy(
                new DefaultRetryPolicy(DEFAULT_FILE_TIMEOUT * 3, DefaultRetryPolicy.DEFAULT_MAX_RETRIES, DefaultRetryPolicy.DEFAULT_BACKOFF_MULT)
        );

        BaseApplication.getInstance().addToRequestQueue(multipartRequest, TAG);
    }

}
