package com.common.framework.util;

import android.Manifest;
import android.app.ActivityManager;
import android.content.ClipData;
import android.content.ClipboardManager;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.graphics.Rect;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Environment;
import android.provider.ContactsContract.CommonDataKinds;
import android.provider.ContactsContract.CommonDataKinds.Email;
import android.provider.ContactsContract.CommonDataKinds.Note;
import android.provider.ContactsContract.CommonDataKinds.Phone;
import android.provider.ContactsContract.Data;
import android.provider.ContactsContract.RawContacts;
import android.provider.Settings;
import android.support.v4.content.ContextCompat;
import android.telephony.TelephonyManager;
import android.text.TextUtils;
import android.util.DisplayMetrics;
import android.view.View;

import java.io.File;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.UUID;

/**
 * app 相关的工具类
 * <p/>
 * Created by chenhongfei on 2016/2/18.
 */
public class AppUtil {

    private static final String TAG = "AppUtil";

    public static int dip2px(Context context, float dipValue) {
        final float scale = context.getResources().getDisplayMetrics().density;
        int px = (int) (dipValue * scale + 0.5f);
//        Log.v("dip2px", "scale:" + scale + " dip:" + dipValue + " px:" + px);
        return px;
    }

    public static int px2dip(Context context, float pxValue) {
        final float scale = context.getResources().getDisplayMetrics().density;
        int dp = (int) (pxValue / scale + 0.5f);
//        Log.v("px2dip", "scale:" + scale + " dip:" + dp + " px:" + pxValue);
        return dp;
    }

    public static int sp2px(Context context, float spValue) {
        final float fontScale = context.getResources().getDisplayMetrics().scaledDensity;
        return (int) (spValue * fontScale + 0.5f);
    }

    /**
     * 获取版本号
     *
     * @param context
     * @return
     */
    public static String getVersionName(Context context) {
        String version = null;
        PackageManager pm = context.getPackageManager();
        try {
            PackageInfo pInfo = pm.getPackageInfo(context.getPackageName(), 0);
            version = pInfo.versionName;

        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
        if (TextUtils.isEmpty(version)) {
            version = "";
        }

        return version;
    }

    public static String getPackageName(Context context) {
        String packageName = null;
        PackageManager pm = context.getPackageManager();
        try {
            PackageInfo pInfo = pm.getPackageInfo(context.getPackageName(), 0);
            packageName = pInfo.packageName;

        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }

        return packageName;
    }

    public static String getCacheDirPath(Context context) {
        if (isExternalStorageMounted()) {
            return getSDCardCacheDirPath(context);
        } else {
            return getPhoneCacheDirPath(context);
        }
    }

    public static String getSDCardCacheDirPath(Context context) {
        String downloadPath = Environment.getExternalStorageDirectory() + "/" + getPackageName(context) + "/";
        File file = new File(downloadPath);
        if (!file.exists()) {
            file.mkdir();
            AspLog.d(TAG, "mkdir " + file.getAbsolutePath());
        } else {
            AspLog.d(TAG, "dir exists " + file.getAbsolutePath());
        }
        return downloadPath;
    }

    public static String getPhoneCacheDirPath(Context context) {
        final PackageManager pm = context.getPackageManager();
        try {// 指定应用缓存文件的目录
            ApplicationInfo ai = pm.getApplicationInfo(context.getPackageName(), 0);
            return ai.dataDir + "/files/";
        } catch (Exception e) {
            return ("/data/data/" + getPackageName(context) + "files/");
        }
    }

    public static boolean isExternalStorageMounted() {
        String state = Environment.getExternalStorageState();
//    	AspLog.d(TAG, "getExternalStorageState = " + state);
        if (state.equals(Environment.MEDIA_MOUNTED)) {
            return true;
        }
        return false;
    }

    /**
     * @param context
     * @return
     */
    public static int getVersionCode(Context context) {
        int version = 0;
        PackageManager pm = context.getPackageManager();
        try {
            PackageInfo pInfo = pm.getPackageInfo(context.getPackageName(), 0);
            version = pInfo.versionCode;

        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }

        return version;
    }

    /**
     * 获取Manifest定义的属性值
     *
     * @param context
     * @param name
     * @return
     */
    public static String getMetaDataValue(Context context, String name) {
        String value = null;
        PackageManager pm = context.getPackageManager();
        try {
            ApplicationInfo appInfo = pm.getApplicationInfo(context.getPackageName(), PackageManager.GET_META_DATA);
            if (appInfo != null && appInfo.metaData != null) {
                value = (String) appInfo.metaData.get(name);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return value;
    }

    /**
     * 获取mac地址
     *
     * @param context
     * @return
     */
    public static String getMacAddress(Context context) {
        WifiManager wifi = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
        WifiInfo info = wifi.getConnectionInfo();
        return info.getMacAddress();
    }

    /**
     * 获取imei号
     *
     * @param context
     * @return
     */
    public static String getDeviceId(Context context) {
        TelephonyManager tm = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);
        return tm.getDeviceId();
    }

    /**
     * 获取imsi
     *
     * @param context
     * @return
     */
    public static String getImsi(Context context) {
        TelephonyManager tm = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);

        String ret = tm.getSubscriberId();
        return ret;
    }

    /**
     * 获取sn号
     *
     * @param context
     * @return
     */
    public static String getSn(Context context) {
        TelephonyManager tm = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);

        return tm.getSimSerialNumber();
    }

    /**
     * 获取android id
     *
     * @param context
     * @return
     */
    public static String getAndroidId(Context context) {
        String androidId = Settings.Secure.getString(context.getContentResolver(), Settings.Secure.ANDROID_ID);

        return androidId;
    }

    /**
     * 生成 uuid
     */
    public static String getUuid() {
        String uuid = java.util.UUID.randomUUID().toString();

        return uuid;
    }


    public static String getPushClientId() {
        try {
            String uuid = java.util.UUID.randomUUID().toString();
            uuid = uuid.replace("-", "");
            uuid += uuid;

            return uuid;
        } catch (Exception e) {

        }

        return "";
    }

    public static boolean isApkInDebug(Context context) {
        try {
            ApplicationInfo info = context.getApplicationInfo();
//			Log.e(TAG, "flags:" + info.flags + " flag debug:" + ApplicationInfo.FLAG_DEBUGGABLE);
//			Log.e(TAG, "flag:" + (info.flags & ApplicationInfo.FLAG_DEBUGGABLE) );
            return (info.flags & ApplicationInfo.FLAG_DEBUGGABLE) != 0;
        } catch (Exception e) {
            return false;
        }
    }

    /**
     * 判断网络是否可用
     *
     * @param context
     * @return
     */
    public static boolean isNetworkAvailable(Context context) {
        ConnectivityManager connectivityManager = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);

        try {
            NetworkInfo wifiNetInfo = connectivityManager.getNetworkInfo(ConnectivityManager.TYPE_WIFI);
            NetworkInfo mobileNetInfo = connectivityManager.getNetworkInfo(ConnectivityManager.TYPE_MOBILE);

            if (wifiNetInfo != null && wifiNetInfo.isConnected()) {
                return true;
            }
            if (mobileNetInfo != null && mobileNetInfo.isConnected()) {
                return true;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return false;
    }

    /**
     * 没有网络
     */
    public static final int NETWORK_TYPE_INVALID = 0;
    /**
     * wap网络
     */
    public static final int NETWORK_TYPE_WAP = 1;
    /**
     * 2G网络
     */
    public static final int NETWORK_TYPE_2G = 2;
    /**
     * 3G网络
     */
    public static final int NETWORK_TYPE_3G = 3;
    /**
     * wifi网络
     */
    public static final int NETWORK_TYPE_WIFI = 4;

    public static int getNetworkType(Context context) {
        int networkType = NETWORK_TYPE_INVALID;
        ConnectivityManager manager = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo networkInfo = manager.getActiveNetworkInfo();

        if (networkInfo != null && networkInfo.isConnected()) {
            int netType = networkInfo.getType();
            if (netType == ConnectivityManager.TYPE_WIFI) {
                networkType = NETWORK_TYPE_WIFI;
            } else if (netType == ConnectivityManager.TYPE_MOBILE) {
                int subtype = networkInfo.getSubtype();
                TelephonyManager mTelephony = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);
                if (subtype == TelephonyManager.NETWORK_TYPE_UMTS && !mTelephony.isNetworkRoaming()) {
                    networkType = NETWORK_TYPE_3G;// 3G
                } else {
                    networkType = NETWORK_TYPE_2G;// 2G
                }
            }
        } else {
            networkType = NETWORK_TYPE_INVALID;
        }

        return networkType;
    }

    public static boolean isWifiNetwork(Context context) {
        if (NETWORK_TYPE_WIFI == getNetworkType(context)) {
            return true;
        }

        return false;
    }

    /**
     * 添加联系人到本机
     *
     * @param context
     * @param contactMap
     * @return
     */
    public static boolean addContact(Context context, Map<String, String> contactMap) {
        try {
            ContentValues values = new ContentValues();

            // 下面的操作会根据RawContacts表中已有的rawContactId使用情况自动生成新联系人的rawContactId
            Uri rawContactUri = context.getContentResolver().insert(RawContacts.CONTENT_URI, values);
            long rawContactId = ContentUris.parseId(rawContactUri);

            // 向data表插入姓名数据
            String name = contactMap.get("name");
            if (name != "") {
                values.clear();
                values.put(RawContacts.Data.RAW_CONTACT_ID, rawContactId);
                values.put(RawContacts.Data.MIMETYPE, CommonDataKinds.StructuredName.CONTENT_ITEM_TYPE);
                values.put(CommonDataKinds.StructuredName.GIVEN_NAME, name);
                context.getContentResolver().insert(Data.CONTENT_URI, values);
            }

            // 向data表插入电话数据
            String mobile_number = contactMap.get("phone");
            if (mobile_number != "") {
                values.clear();
                values.put(Data.RAW_CONTACT_ID, rawContactId);
                values.put(Data.MIMETYPE, Phone.CONTENT_ITEM_TYPE);
                values.put(Phone.NUMBER, mobile_number);
                values.put(Phone.TYPE, Phone.TYPE_MOBILE);
                context.getContentResolver().insert(Data.CONTENT_URI, values);
            }

            // 向data表插入Email数据
            String email = contactMap.get("email");
            if (email != "") {
                values.clear();
                values.put(Data.RAW_CONTACT_ID, rawContactId);
                values.put(Data.MIMETYPE, Email.CONTENT_ITEM_TYPE);
                values.put(Email.DATA, email);
                values.put(Email.TYPE, Email.TYPE_WORK);
                context.getContentResolver().insert(Data.CONTENT_URI, values);
            }

            // 向data表插入备注信息
            String describe = contactMap.get("remark");
            if (describe != "") {
                values.clear();
                values.put(Data.RAW_CONTACT_ID, rawContactId);
                values.put(Data.MIMETYPE, Note.CONTENT_ITEM_TYPE);
                values.put(Note.NOTE, describe);
                context.getContentResolver().insert(Data.CONTENT_URI, values);
            }

        } catch (Exception e) {
            return false;
        }

        return true;
    }

    /**
     * 复制到剪切板
     *
     * @param text
     */
    public static void copyClipboard(String text, Context context) {
        ClipboardManager cbManager = (ClipboardManager) context.getSystemService(Context.CLIPBOARD_SERVICE);
        cbManager.setPrimaryClip(ClipData.newPlainText(null, text.trim()));
    }

    public static boolean isIntentAvailable(Context context, final Intent intent) {
        final PackageManager packageManager = context.getPackageManager();

        List<ResolveInfo> list = packageManager.queryIntentActivities(intent, PackageManager.MATCH_DEFAULT_ONLY);
        return list.size() > 0;
    }

    /**
     * 根据进程pid获取进程名
     *
     * @param context
     * @param pid
     * @return
     */
    public static String getProcessName(Context context, int pid) {
        String processName = null;
        ActivityManager am = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
        List<ActivityManager.RunningAppProcessInfo> runningProcessInfoList = am.getRunningAppProcesses();
        if (null != runningProcessInfoList && !runningProcessInfoList.isEmpty()) {
            for (ActivityManager.RunningAppProcessInfo processInfo : runningProcessInfoList) {
                if (pid == processInfo.pid) {
                    processName = processInfo.processName;
                    break;
                }
            }
        }

        return processName;
    }

    public static boolean isMainProcess(Context context) {
        String packageName = getPackageName(context);
        String processName = getProcessName(context, android.os.Process.myPid());
//        AspLog.d("jsonjson", "packageName:" + packageName + "   processName:" + processName + " " + packageName.equals(processName));
        if (packageName.equals(processName)) {
            return true;
        }

        return false;
    }

    /**
     * 键盘是否显示
     *
     * @param rootView
     * @return
     */
    public boolean isSoftInputShown(View rootView) {
        final int softKeyboardHeight = 100;
        Rect r = new Rect();
        rootView.getWindowVisibleDisplayFrame(r);
        DisplayMetrics dm = rootView.getResources().getDisplayMetrics();
        int heightDiff = rootView.getBottom() - r.bottom;
        return heightDiff > softKeyboardHeight * dm.density;
    }

    public static String getUUID(Context context) {
        final TelephonyManager tm = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);
        final String tmDevice, tmSerial, androidId;
        int checkPermission = ContextCompat.checkSelfPermission(context, Manifest.permission.READ_PHONE_STATE);
        if (checkPermission == PackageManager.PERMISSION_GRANTED) {
            tmDevice = "" + tm.getDeviceId();
            tmSerial = "" + tm.getSimSerialNumber();
        } else {
            tmDevice = "";
            tmSerial = "";
        }

        androidId = "" + getAndroidId(context);
        UUID deviceUuid = new UUID(androidId.hashCode(), ((long) tmDevice.hashCode() << 32) | tmSerial.hashCode());
        String uuid = deviceUuid.toString();
        int index = uuid.lastIndexOf('-') + 1;
        if (index > 1) {
            return uuid.substring(index);
        }
        return uuid.substring(uuid.length() - 12);
    }

    public static String getCountry(Context context) {
        Locale locale = context.getResources().getConfiguration().locale;
        String country = locale.getCountry();

        return country;
    }

    public static String getLanguage(Context context) {
        Locale locale = context.getResources().getConfiguration().locale;
        String language = locale.getLanguage();

        return language;
    }

}
