package com.common.framework.util;

import android.os.Environment;
import android.util.Log;

import com.common.app.base.BaseApplication;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * 日志打印类，提供了一个开关来决定是否打印日志
 */
public class AspLog {
    private static boolean isFirst = true;

    // 是否强制打开log
    private static boolean mIsForceLog = false;
    public static boolean isPrintLog = true;
    public static boolean isPrintSDcardLog = false;
    public static boolean isWriteToFile = false;
    private final static String LOG_FILEPATH = BaseApplication.getInstance().getPackageName() + File.separator + "log" + File.separator;
    private final static String LOG_FILENAME = "log";
    private final static String LOG_FILEEXT = ".txt";
    private static File mLogFile;

    private final static long LOGFILE_LIMIT = 1000000L;
    private final static SimpleDateFormat DATEFORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    //    private final static SimpleDateFormat DATEFORMAT1 = new SimpleDateFormat("yyyyMMddHHmmss");
    private static String mSdcardRootPath = "/sdcard/";

    private static void checkLog() {
        createLogFile();
    }

    private static void createLogFile() {
        if (isWriteToFile) {
            synchronized (LOG_FILENAME) {
                if (mLogFile == null) {
                    try {
                        if (!Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
                            return;
                        }

                        // File sdcardRootPath =
                        // Environment.getExternalStorageDirectory();
                        mSdcardRootPath = Environment.getExternalStorageDirectory().getAbsolutePath() + File.separator;

                        // File cfgfile = new File(mSdcardRootPath +
                        // LOG_CFGFILE);
                        // if(!cfgfile.exists()){//没用开启配置文件则不写日志文件
                        // return;
                        // }

                        File logpath = new File(mSdcardRootPath + LOG_FILEPATH);
                        if (!logpath.exists()) {
                            logpath.mkdir();
                        }

                        mLogFile = new File(mSdcardRootPath + LOG_FILEPATH + LOG_FILENAME + LOG_FILEEXT);
                        if (!mLogFile.exists()) {
                            AspLog.d("TestFile", "Create the file:" + LOG_FILENAME);
                            mLogFile.createNewFile();
                        }

                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                } else {
                    if (mLogFile.isFile()) {
                        if (mLogFile.length() > LOGFILE_LIMIT) {
                            StringBuffer sb = new StringBuffer(mSdcardRootPath);
                            sb.append(LOG_FILEPATH);
                            sb.append(LOG_FILENAME);
                            sb.append(DATEFORMAT.format(new Date()));
                            sb.append(LOG_FILEEXT);
                            mLogFile.renameTo(new File(sb.toString()));
                            sb = null;
                            sb = new StringBuffer(mSdcardRootPath);
                            sb.append(LOG_FILEPATH);
                            sb.append(LOG_FILENAME);
                            sb.append(LOG_FILEEXT);
                            mLogFile = new File(sb.toString());
                            sb = null;
                            if (!mLogFile.exists()) {
                                AspLog.d("TestFile", "Create the file:" + LOG_FILENAME + LOG_FILEEXT);
                                try {
                                    mLogFile.createNewFile();
                                } catch (IOException e) {
                                    e.printStackTrace();
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    private static void writeLogFile(String level, String tag, String msg) {
        if (isWriteToFile) {
            synchronized (LOG_FILENAME) {
                if (mLogFile != null) {

                    StringBuffer sb = new StringBuffer();
                    sb.append(DATEFORMAT.format(new Date()));
                    sb.append(": ");
                    sb.append(level);
                    sb.append(": ");
                    sb.append(tag);
                    sb.append(": ");
                    sb.append(msg);
                    sb.append("\n");
                    RandomAccessFile raf = null;
                    try {
                        raf = new RandomAccessFile(mLogFile, "rw");
                        raf.seek(mLogFile.length());
                        raf.write(sb.toString().getBytes("UTF-8"));
                    } catch (IOException e) {
                        e.printStackTrace();

                    } finally {
                        sb = null;
                        if (raf != null) {
                            try {
                                raf.close();
                            } catch (IOException e) {
                                // TODO Auto-generated catch block
                                e.printStackTrace();
                            }
                        }

                    }
                }
            }
        }
    }

    public static void v(String tag, String msg) {
        if (!isLoggable(Log.VERBOSE)) {
            return;
        }
        checkLog();
        if (isPrintLog) {
            Log.v(tag, msg == null ? "" : msg);
        }
        writeLogFile("VERBOSE", tag, msg);
    }

    public static void v(String tag, String msg, Throwable tr) {
        if (!isLoggable(Log.VERBOSE)) {
            return;
        }
        checkLog();
        if (isPrintLog) {
            Log.v(tag, msg == null ? "" : msg, tr);
        }
        writeLogFile("VERBOSE", tag, msg);
    }

    public static void d(String tag, String msg) {
        if (!isLoggable(Log.DEBUG)) {
            return;
        }
        checkLog();
        if (isPrintLog) {
            Log.d(tag, msg == null ? "" : msg);
        }
        writeLogFile("DEBUG", tag, msg);
    }

    public static void d(String tag, String msg, Throwable tr) {
        if (!isLoggable(Log.DEBUG)) {
            return;
        }
        checkLog();
        if (isPrintLog) {
            Log.d(tag, msg == null ? "" : msg, tr);
        }
        writeLogFile("DEBUG", tag, msg);
    }

    public static void i(String tag, String msg) {
        if (!isLoggable(Log.INFO)) {
            return;
        }
        checkLog();
        if (isPrintLog) {
            Log.i(tag, msg == null ? "" : msg);
        }

        writeLogFile("INFO", tag, msg);
    }

    public static void i(String tag, String msg, Throwable tr) {
        if (!isLoggable(Log.INFO)) {
            return;
        }
        checkLog();
        if (isPrintLog) {
            Log.i(tag, msg == null ? "" : msg, tr);
        }
        writeLogFile("INFO", tag, msg);
    }

    public static void w(String tag, String msg) {
        if (!isLoggable(Log.WARN)) {
            return;
        }
        checkLog();
        if (isPrintLog) {
            Log.w(tag, msg == null ? "" : msg);
        }
        writeLogFile("WARN", tag, msg);
    }

    public static void w(String tag, String msg, Throwable tr) {
        if (!isLoggable(Log.WARN)) {
            return;
        }
        checkLog();
        if (isPrintLog) {
            Log.w(tag, msg == null ? "" : msg, tr);
        }
        writeLogFile("WARN", tag, msg);
    }

    public static void e(String tag, String msg) {
        if (!isLoggable(Log.ERROR)) {
            return;
        }
        checkLog();
        if (isPrintLog) {
            Log.e(tag, msg == null ? "" : msg);
        }
        writeLogFile("ERROR", tag, msg);
    }

    public static void e(String tag, String msg, Throwable tr) {
        if (!isLoggable(Log.ERROR)) {
            return;
        }
        checkLog();
        if (isPrintLog) {
            Log.e(tag, msg == null ? "" : msg, tr);
        }
        writeLogFile("ERROR", tag, msg);
    }

    /**
     * 是否打印log
     * <p>
     * 1、mIsForceLog 强制打开log，记录所有的log
     * 2、mIsForceLog 为false， （1）debug状态下，记录所有的log（2）在release下，只记录 >= warn 的log
     *
     * @return
     */
    private static boolean isLoggable(int logLevel) {
        if (mIsForceLog) {
            return true;
        }
        boolean isInDebug = AppUtil.isApkInDebug(BaseApplication.getInstance());
        if (isInDebug) {
            return true;
        }
        return logLevel >= Log.WARN;
    }
}
