package com.common.framework.util;

import android.text.TextUtils;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * 时间工具类
 * Created by chenhongfei on 2015/8/17.
 */
public class DatetimeUtil {

    /**
     * 把时间戳转成标准的datetime 格式 YYYY-MM-dd HH:mm:ss
     *
     * @param time
     * @return
     */
    public static String getFormatDatetime(long time) {
        String datetime = "";
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            datetime = sdf.format(new Date(time));
        } catch (Exception e) {
            e.printStackTrace();
        }

        return datetime;
    }

    /**
     * 自定义日期时间格式
     *
     * @param time
     * @param format
     * @return
     */
    public static String getFormatDatetime(long time, String format) {
        String datetime = "";
        try {
            SimpleDateFormat sdf = new SimpleDateFormat(format);
            datetime = sdf.format(new Date(time));
        } catch (Exception e) {
            e.printStackTrace();
        }

        return datetime;
    }

    /**
     * 自定义当前日期时间格式
     *
     * @param format
     * @return
     */
    public static String getCurrentFormatDatetime(String format) {
        String datetime = "";
        try {
            Date currentDate = new Date();
            SimpleDateFormat sdf = new SimpleDateFormat(format);
            datetime = sdf.format(currentDate);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return datetime;
    }

    /**
     * 获取当前的日期时间
     *
     * @return
     */
    public static String getCurrentDatetime() {
        String datetime = "";
        try {
            Date currentDate = new Date();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            datetime = sdf.format(currentDate);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return datetime;
    }

    /**
     * 获取当前的日期时间,精确到毫秒
     *
     * @return
     */
    public static String getCurrentDatetimeMilli() {
        return getCurrentFormatDatetime("yyyy-MM-dd HH:mm:ss.S");
    }


    /**
     * 获取明天的中文短时间格式 ：yyyy 年 mm 月 dd 日 00：00
     *
     * @return
     */
    public static String getNextDayShortCHDatetime() {
        Calendar calendar = Calendar.getInstance();
        String shortDatetime = calendar.get(Calendar.YEAR) + "年" + (calendar.get(Calendar.MONTH) + 1) + "月" + (calendar.get(Calendar.DAY_OF_MONTH) + 1) + "日"
                + " " + calendar.get(Calendar.HOUR_OF_DAY) + ":" + calendar.get(Calendar.MINUTE);

        return shortDatetime;
    }

    /**
     * yyyy年MM月dd日 HH:mm 转成timestamp 单位是毫秒
     *
     * @param strDatetime
     * @return
     */
    public static long datetimeToTimestamp(String strDatetime, String format) {
        long timestamp = 0;
        try {
            SimpleDateFormat sdf = new SimpleDateFormat(format);
            Date d = sdf.parse(strDatetime);
            timestamp = d.getTime();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return timestamp;
    }

    public static long datetimeToTimestamp(String strDatetime) {
        return datetimeToTimestamp(strDatetime, "yyyy-MM-dd HH:mm:ss");
    }

    /**
     * 获取二个日期时间的间隔时间 ，单位毫秒
     */
    public static long getTimeInterval(String datetime1, String datetime2) {
        long interval = 0;
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.S");
        try {
            Date date1 = sdf.parse(datetime1);
            Date date2 = sdf.parse(datetime2);
            interval = Math.abs((date1.getTime() - date2.getTime()));
        } catch (Exception e) {
            e.printStackTrace();
            return 0;
        }

        return interval;
    }

    /**
     * 能力评估，格式化日期
     *
     * @param datetime
     * @return
     */
    public static String formatAbilityForecastDate(String datetime) {
        String formatRet = "";
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        SimpleDateFormat sdfResult = new SimpleDateFormat("MM-dd");
        try {
            Date date = sdf.parse(datetime);
            formatRet = sdfResult.format(date);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return formatRet;
    }

    public static boolean isToday(long time) {
        boolean result = false;
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            String today = sdf.format(new Date());
            String date = sdf.format(new Date(time));
            result = today.equals(date);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return result;
    }

    public static boolean isYesterday(long time) {
        boolean result = false;
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            String yesterday = sdf.format(new Date(System.currentTimeMillis() - 24 * 3600 * 1000));
            String date = sdf.format(new Date(time));

            result = yesterday.equals(date);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return result;
    }

    public static long HMS2Time(String datetime) {
        long timestamp = 0L;
        if (TextUtils.isEmpty(datetime)) {
            return timestamp;
        }
        String[] tArray = datetime.split(":");
        if (null == tArray || tArray.length <= 0) {
            return timestamp;
        }
        if (tArray.length >= 1) {
            timestamp += 3600 * Integer.parseInt(tArray[0]);
        }
        if (tArray.length >= 2) {
            timestamp += 60 * Integer.parseInt(tArray[1]);
        }
        if (tArray.length >= 3) {
            timestamp += Integer.parseInt(tArray[2]);
        }

        return timestamp;
    }

    // 获得当前日期与本周一相差的天数
    private static int getMondayPlus() {
        Calendar cd = Calendar.getInstance();
        // 获得今天是一周的第几天，星期日是第一天，星期二是第二天......
        int dayOfWeek = cd.get(Calendar.DAY_OF_WEEK);
        if (dayOfWeek == 1) {
            return -6;
        } else {
            return 2 - dayOfWeek;
        }
    }

    /**
     * i 为了0，当前周一，i为6，当前周日
     *
     * @param i
     * @return
     */
    public static String getCurrentFormatWeekDate(int i, String format) {
        int mondayPlus = getMondayPlus();
        GregorianCalendar currentDate = new GregorianCalendar();
        currentDate.add(GregorianCalendar.DATE, mondayPlus + i);
        Date monday = currentDate.getTime();
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        String weekDate = sdf.format(monday);
        return weekDate;
    }

    public static long getWeekDayTime(int i) {
        int mondayPlus = getMondayPlus();
        GregorianCalendar currentDate = new GregorianCalendar();
        currentDate.add(GregorianCalendar.DATE, mondayPlus + i);
        Date date = currentDate.getTime();
        long timestamp = date.getTime();
        return timestamp;
    }

    public static int getCurrentWeekDay() {
        Calendar cd = Calendar.getInstance();
        // 获得今天是一周的第几天，星期日是第一天，星期二是第二天......
        int weekDay = cd.get(Calendar.DAY_OF_WEEK) - 1;

        return weekDay;
    }

    public static boolean isInTwoTime(String datetime, String minDatetime, String maxDatetime, String format) {
        long time = datetimeToTimestamp(datetime, format);
        long minTime = datetimeToTimestamp(minDatetime, format);
        long maxTime = datetimeToTimestamp(maxDatetime, format);
        if (time >= minTime && time <= maxTime) {
            return true;
        }

        return false;
    }

    public static String toCHWeekDay(int weekDay) {
        String cnWeekDay = "";
        switch (weekDay) {
            case 0:
                cnWeekDay = "日";
                break;
            case 1:
                cnWeekDay = "一";
                break;
            case 2:
                cnWeekDay = "二";
                break;
            case 3:
                cnWeekDay = "三";
                break;
            case 4:
                cnWeekDay = "四";
                break;
            case 5:
                cnWeekDay = "五";
                break;
            case 6:
                cnWeekDay = "六";
                break;
        }

        return cnWeekDay;
    }
}
