package com.common.framework.util;

import android.content.Context;

import org.apache.http.util.EncodingUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by chenhongfei on 2015/11/9.
 */
public class FileUtils {
    private static final String TAG = "FileUtils";
    private static final String ENCODING = "UTF-8";

    /**
     * 读取文件内容
     *
     * @param context
     * @param fileName
     * @return
     */
    public static String readFileData(Context context, String fileName) {
        String jsonString = "";
        FileInputStream fInputStream = null;
        try {
            fInputStream = context.openFileInput(fileName);
            int length = fInputStream.available();
            byte[] buffer = new byte[length];
            fInputStream.read(buffer);
            jsonString = new String(buffer, ENCODING);
        } catch (FileNotFoundException e) {
            AspLog.w(TAG, fileName + " open failed: no such file");
        } catch (IOException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (fInputStream != null) {
                    fInputStream.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

        }

        return jsonString;
    }

    /**
     * 把字符串写进文件
     *
     * @param context
     * @param fileName
     * @param data
     * @return
     */
    public static boolean writeFileData(Context context, String fileName, String data) {
        FileOutputStream fOutputStream = null;
        boolean result = false;
        try {
            fOutputStream = context.openFileOutput(fileName, Context.MODE_PRIVATE);
            byte[] bytes = data.getBytes(ENCODING);
            fOutputStream.write(bytes);

            result = true;
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (fOutputStream != null) {
                    fOutputStream.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return result;
    }

    public static boolean deleteFile(Context context, String fileName) {
        return  context.deleteFile(fileName);
    }

    public static List<File> getFileList(String dirPath) {
        List<File> fileList = new ArrayList<>();
        File[] allFiles = new File(dirPath).listFiles();
        for (int i = 0; i < allFiles.length; i++) {
            File file = allFiles[i];

            file.hashCode();
            if (file.isFile()) {
                fileList.add(file);
            }
        }

        return fileList;
    }


    public static String readAssets(Context context, String fileName) {
        String jsonString = "";
        InputStream is = null;
        AspLog.d(TAG, "fileName:" + fileName);
        try {
            is = context.getResources().getAssets().open(fileName);

            int length = is.available();
            byte[] buffer = new byte[length];
            is.read(buffer);
            jsonString = new String(buffer, ENCODING);
        } catch (FileNotFoundException e) {
            AspLog.w(TAG, fileName + " open failed: no such file");
        } catch (IOException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (is != null) {
                    is.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

        }

        return jsonString;
    }

    //读文件
    public static String readSDFile(String fileName) throws IOException {

        File file = new File(fileName);

        FileInputStream fis = new FileInputStream(file);

        int length = fis.available();

        byte [] buffer = new byte[length];
        fis.read(buffer);

        String res = EncodingUtils.getString(buffer, "UTF-8");

        fis.close();
        return res;
    }

    //写文件
    public static void writeSDFile(String fileName, String write_str) throws IOException{

        File file = new File(fileName);

        FileOutputStream fos = new FileOutputStream(file);

        byte [] bytes = write_str.getBytes();

        fos.write(bytes);

        fos.close();
    }
}
