package com.common.framework.util;

import android.content.Context;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.UUID;

/**
 * 每个客户端生成一个唯一的uuid保存在本地，
 * 当删除app重装的时候，本地文件被删除，会重新生成
 * <p/>
 * Created by chenhongfei on 2015/11/16.
 */
public class Installation {
    private static String sUuid = null;
    private static final String INSTALLATION = "INSTALLATION";


    public synchronized static String getUuid(Context context) {
        if (context == null) {
            return "";
        }
        if (sUuid == null) {
            File installation = new File(context.getFilesDir(), INSTALLATION);
            try {
                if (!installation.exists()) {
                    sUuid = writeInstallationFile(installation);
                } else {
                    sUuid = readInstallationFile(installation);
                }
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }

        return sUuid;
    }

    /**
     * 读文件
     *
     * @param installation
     * @return
     * @throws IOException
     */
    private static String readInstallationFile(File installation) throws IOException {
        RandomAccessFile f = new RandomAccessFile(installation, "r");
        byte[] bytes = new byte[(int) f.length()];
        f.readFully(bytes);
        f.close();

        return new String(bytes);
    }

    /**
     * 写文件
     *
     * @param installation
     * @return
     * @throws IOException
     */
    private static String writeInstallationFile(File installation) throws IOException {
        FileOutputStream out = new FileOutputStream(installation);
        String uuid = generateId();
        out.write(uuid.getBytes());
        out.close();

        return uuid;
    }

    /**
     * 根据服务端要求，生成64位的id
     *
     * @return
     */
    private static String generateId() {
        String uuid = UUID.randomUUID().toString();

        String clientId = uuid.replace("-", "");
        clientId += clientId;

        return clientId;
    }

}
