package com.common.framework.util;

import android.text.TextUtils;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Created by chenhongfei on 2016/2/19.
 */
public class StringUtil {
    /**
     * 判断字符串是否包含html标签
     *
     * @param string
     * @return
     */
    public static boolean isContainHtml(String string) {
        Pattern pattern = Pattern.compile("<(\\S*?) [^>]*>.*?</\\1>|<[^>]*? />|<.*?>");
        Matcher matcher = pattern.matcher(string);
        boolean result = matcher.find();

        return result;
    }

    /**
     * 半角转全角
     *
     * @param string
     * @return
     */
    public static String toSBC(String string) {
        char[] c = string.toCharArray();
        for (int i = 0; i < c.length; i++) {
            if (c[i] == 32) {
                c[i] = (char) 12288;
                continue;
            }
            if (c[i] < 127)
                c[i] = (char) (c[i] + 65248);
        }
        return new String(c);
    }

    /**
     * 全角转换为半角
     *
     * @param string
     * @return
     */
    public static String toDBC(String string) {
        char[] c = string.toCharArray();
        for (int i = 0; i < c.length; i++) {
            // 全角空格为12288，半角空格为32
            if (c[i] == 12288) {
                c[i] = (char) 32;
                continue;
            }
            // 其他字符半角(33-126)与全角(65281-65374)的对应关系是：均相差65248
            if (c[i] > 65280 && c[i] < 65375)
                c[i] = (char) (c[i] - 65248);
        }
        return new String(c);
    }

    /**
     * 只转义指定的字符
     *
     * @param string
     * @return
     */
    public static String toSpecialSBC(String string) {
        char[] c = string.toCharArray();
        for (int i = 0; i < c.length; i++) {
            if (c[i] == 32) {
                c[i] = (char) 12288;
                continue;
            }
            if (c[i] >= 48 && c[i] <= 57) // 只转义数字字符，会计的字符基本上都是数字字符（大题有换行符\n，被转义了会乱码）
                c[i] = (char) (c[i] + 65248);
        }
        return new String(c);
    }

    /**
     * stream 转 String
     *
     * @param in
     * @param encoding
     * @return
     */
    public static String inputStreamToString(InputStream in, String encoding) {
        final int BUFFER_SIZE = 4096;
        String retString = null;
        try {
            ByteArrayOutputStream outStream = new ByteArrayOutputStream();
            byte[] data = new byte[BUFFER_SIZE];
            int count = -1;
            while ((count = in.read(data, 0, BUFFER_SIZE)) != -1)
                outStream.write(data, 0, count);

            data = null;
            retString = new String(outStream.toByteArray(), encoding == null ? "UTF-8" : encoding);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return retString;
    }

    /**
     * 转成标准的http url
     *
     * @param url
     * @return
     */
    public static String formatUrl(String url) {
        if (TextUtils.isEmpty(url)) {
            return url;
        }
        if (url.startsWith("http://") || url.startsWith("https://") || url.startsWith("file://")) {
            return url;
        }

        String formatUrl = "http://" + url;

        return formatUrl;
    }

    /**
     * 格式化小数点（如果小数位为0，则去除）
     *
     * @param number
     * @return
     */
    public static String formatZeroDecimalPoint(double number) {
        // 判断有没小数部分
        int intNumber = (int) number;
        double point = number - intNumber;
        if (point > 0) {
            return "" + number;
        } else {
            return "" + intNumber;
        }
    }

    /**
     * 判断手机格式是否正确，改为简单判断
     *
     * @param mobile
     * @return
     */
    public static boolean isMobile(String mobile) {

        // Pattern p = Pattern
        // .compile("^((13[0-9])|(15[^4,\\D])|(18[0,5-9]))\\d{8}$");
        // Matcher m = p.matcher(mobiles);
        // return m.matches();
        //
        if (mobile.startsWith("1") && mobile.length() == 11) {
            return true;
        }
        return false;
    }

    /**
     * 把小数点格式化到length位
     *
     * @param number
     * @param length
     * @return
     */
    public static String formatDecimal(double number, int length) {
        // 判断有没小数部分
        int p = (int) Math.pow(10, length);
        double num = (double) (Math.round(number * p)) / p;
        String retString = formatZeroDecimalPoint(num);

        return retString;
    }

    public static String random() {
        String random = String.valueOf((int) (Math.random() * 1000)); // 产生随机数

        return random;
    }
}
