package com.common.framework.view.customview;

import android.annotation.SuppressLint;
import android.content.Context;
import android.util.AttributeSet;
import android.widget.Chronometer;

import java.text.SimpleDateFormat;

/**
 * Created by chenhongfei on 2016/9/29.
 */
@SuppressLint({"ViewConstructor", "SimpleDateFormat"})
public class CounterChronometer extends Chronometer {

    private long mCriticalTime = 5;
    private boolean mIsCountDown = true;
    private long mNextTime;
    private OnTimerListener mListener;
    private SimpleDateFormat mTimeFormat;

    public CounterChronometer(Context context) {
        super(context);
    }

    public CounterChronometer(Context context, AttributeSet attrs) {
        super(context, attrs);
        // TODO 自动生成的构造函数存根
        mTimeFormat = new SimpleDateFormat("HH:mm:ss");
        this.setOnChronometerTickListener(listener);
    }

    /**
     * 设置时间格式
     *
     * @param pattern 计时格式
     */
    public void setTimeFormat(String pattern, boolean isCountDown) {
        mTimeFormat = new SimpleDateFormat(pattern);
        mIsCountDown = isCountDown;
    }

    public void setOnTimeCompleteListener(OnTimerListener l) {
        mListener = l;
    }

    OnChronometerTickListener listener = new OnChronometerTickListener() {
        @Override
        public void onChronometerTick(Chronometer chronometer) {
            if (mIsCountDown) {
                if (mNextTime == mCriticalTime) {
                    if (null != mListener) {
                        mListener.onCritical();
                    }
                }
                if (mNextTime <= 0) {
                    if (mNextTime == 0) {
                        CounterChronometer.this.stop();
                        if (null != mListener)
                            mListener.onTimeComplete();
                    }
                    mNextTime = 0;
                    updateTimeText();
                    return;
                }

                mNextTime--;
            } else {
                mNextTime++;
            }

            updateTimeText();
        }
    };

    /**
     * 初始化时间
     *
     * @param time
     */
    public void initTime(long time) {
        mNextTime = time;
        updateTimeText();
    }

    private void updateTimeText() {
        this.setText(formatTime(mNextTime));
    }

    public interface OnTimerListener {
        void onTimeComplete();

        void onCritical();
    }

    public String formatTime(long time) {
        String h = time / 3600 > 9 ? "" + time / 3600 : "0" + time / 3600;
        String m = (time % 3600) / 60 > 9 ? "" + (time % 3600) / 60 : "0" + (time % 3600) / 60;
        String s = (time % 60) > 9 ? "" + (time % 60) : "0" + (time % 60);

        String ret = String.format("%s:%s:%s", h, m, s);
//        AspLog.v("jsonjson", ret);
        return ret;
    }
}



