package com.common.framework.view.customview;

import android.content.Context;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;
import android.widget.LinearLayout;

/**
 * Created by chenhongfei on 2017/2/7.
 */
public class DragLinearLayout extends LinearLayout implements View.OnTouchListener {
    protected int screenWidth;
    protected int screenHeight;
    protected int lastX;
    protected int lastY;
    private int oriLeft;
    private int oriRight;
    private int oriTop;
    private int oriBottom;
    private int dragDirection;
    private static final int TOP = 0x15;
    private static final int LEFT = 0x16;
    private static final int BOTTOM = 0x17;
    private static final int RIGHT = 0x18;
    private static final int LEFT_TOP = 0x11;
    private static final int RIGHT_TOP = 0x12;
    private static final int LEFT_BOTTOM = 0x13;
    private static final int RIGHT_BOTTOM = 0x14;
    private static final int CENTER = 0x19;
    private int offset = 20;

    private int mDragDirection = 0;
    private static final int DRAG_OFFSET = 60;
    private static final int MIN_HEIGHT = 400;
    private static final int MIN_WIDTH = 400;


    protected void initScreenW_H() {
        screenHeight = getResources().getDisplayMetrics().heightPixels - 40;
        screenWidth = getResources().getDisplayMetrics().widthPixels;
    }

    public DragLinearLayout(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        setOnTouchListener(this);
        initScreenW_H();
    }

    public DragLinearLayout(Context context, AttributeSet attrs) {
        super(context, attrs);
        setOnTouchListener(this);
        initScreenW_H();
    }

    public DragLinearLayout(Context context) {
        super(context);
        setOnTouchListener(this);
        initScreenW_H();
    }

//    @Override
//    protected void onDraw(Canvas canvas) {
//        super.onDraw(canvas);
//        paint.setColor(Color.RED);
//        paint.setStrokeWidth(4.0f);
//        paint.setStyle(Style.STROKE);
//        canvas.drawRect(offset, offset, getWidth() - offset, getHeight()
//                - offset, paint);
//    }

    @Override
    public boolean onTouch(View v, MotionEvent event) {
        int action = event.getAction();
        if (action == MotionEvent.ACTION_DOWN) {
            oriLeft = v.getLeft();
            oriRight = v.getRight();
            oriTop = v.getTop();
            oriBottom = v.getBottom();
            lastY = (int) event.getRawY();
            lastX = (int) event.getRawX();
            dragDirection = getDirection(v, (int) event.getX(),
                    (int) event.getY());
        }
        // 处理拖动事件
        delDrag(v, event, action);
        invalidate();
        int childCount = getChildCount();
        for (int i = 0; i < childCount; i ++) {
            View childView = getChildAt(i);
            childView.invalidate();
        }
        return false;
    }


    protected void delDrag(View v, MotionEvent event, int action) {
        switch (action) {
            case MotionEvent.ACTION_MOVE:
                int dx = (int) event.getRawX() - lastX;
                int dy = (int) event.getRawY() - lastY;
                switch (dragDirection) {
                    case LEFT: // 左边缘
                        left(v, dx);
                        break;
                    case RIGHT: // 右边缘
                        right(v, dx);
                        break;
                    case BOTTOM: // 下边缘
                        bottom(v, dy);
                        break;
                    case TOP: // 上边缘
                        top(v, dy);
                        break;
                    case CENTER: // 点击中心-->>移动
//                        center(v, dx, dy);
                        break;
                    case LEFT_BOTTOM: // 左下
                        left(v, dx);
                        bottom(v, dy);
                        break;
                    case LEFT_TOP: // 左上
                        left(v, dx);
                        top(v, dy);
                        break;
                    case RIGHT_BOTTOM: // 右下
                        right(v, dx);
                        bottom(v, dy);
                        break;
                    case RIGHT_TOP: // 右上
                        right(v, dx);
                        top(v, dy);
                        break;
                }
                if (dragDirection != CENTER && dragDirection == mDragDirection) {
                    v.layout(oriLeft, oriTop, oriRight, oriBottom);
                }
                lastX = (int) event.getRawX();
                lastY = (int) event.getRawY();
                break;
            case MotionEvent.ACTION_UP:
                dragDirection = 0;
                break;
        }
    }


    private void center(View v, int dx, int dy) {
        int left = v.getLeft() + dx;
        int top = v.getTop() + dy;
        int right = v.getRight() + dx;
        int bottom = v.getBottom() + dy;
        if (left < -offset) {
            left = -offset;
            right = left + v.getWidth();
        }
        if (right > screenWidth + offset) {
            right = screenWidth + offset;
            left = right - v.getWidth();
        }
        if (top < -offset) {
            top = -offset;
            bottom = top + v.getHeight();
        }
        if (bottom > screenHeight + offset) {
            bottom = screenHeight + offset;
            top = bottom - v.getHeight();
        }
        v.layout(left, top, right, bottom);
    }


    private void top(View v, int dy) {
        oriTop += dy;
        if (oriTop < -offset) {
            oriTop = -offset;
        }
        if (oriBottom - oriTop - 2 * offset < MIN_HEIGHT) {
            oriTop = oriBottom - 2 * offset - MIN_HEIGHT;
        }
    }


    private void bottom(View v, int dy) {
        oriBottom += dy;
        if (oriBottom > screenHeight + offset) {
            oriBottom = screenHeight + offset;
        }
        if (oriBottom - oriTop - 2 * offset < MIN_HEIGHT) {
            oriBottom = MIN_HEIGHT + oriTop + 2 * offset;
        }
    }


    private void right(View v, int dx) {
        oriRight += dx;
        if (oriRight > screenWidth + offset) {
            oriRight = screenWidth + offset;
        }
        if (oriRight - oriLeft - 2 * offset < MIN_WIDTH) {
            oriRight = oriLeft + 2 * offset + MIN_WIDTH;
        }
    }


    private void left(View v, int dx) {
        oriLeft += dx;
        if (oriLeft < -offset) {
            oriLeft = -offset;
        }
        if (oriRight - oriLeft - 2 * offset < MIN_WIDTH) {
            oriLeft = oriRight - 2 * offset - MIN_WIDTH;
        }
    }


    protected int getDirection(View v, int x, int y) {
        int left = v.getLeft();
        int right = v.getRight();
        int bottom = v.getBottom();
        int top = v.getTop();
        if (x < DRAG_OFFSET && y < DRAG_OFFSET) {
            return LEFT_TOP;
        }
        if (y < DRAG_OFFSET && right - left - x < DRAG_OFFSET) {
            return RIGHT_TOP;
        }
        if (x < DRAG_OFFSET && bottom - top - y < DRAG_OFFSET) {
            return LEFT_BOTTOM;
        }
        if (right - left - x < DRAG_OFFSET && bottom - top - y < DRAG_OFFSET) {
            return RIGHT_BOTTOM;
        }
        if (x < DRAG_OFFSET) {
            return LEFT;
        }
        if (y < DRAG_OFFSET) {
            return TOP;
        }
        if (right - left - x < DRAG_OFFSET) {
            return RIGHT;
        }
        if (bottom - top - y < DRAG_OFFSET) {
            return BOTTOM;
        }
        return CENTER;
    }

    public int getCutWidth() {
        return getWidth() - 2 * offset;
    }


    public int getCutHeight() {
        return getHeight() - 2 * offset;
    }
}