package com.common.framework.view.customview;

import android.content.Context;
import android.content.res.TypedArray;
import android.support.v7.widget.AppCompatEditText;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.AttributeSet;


import com.common.app.R;

import java.util.LinkedList;
import java.util.List;

/**
 * Created by guqingbo on 2016-08-12.
 */
public class FilterEditText extends AppCompatEditText {
    private int maxNum;
    private List<IJugeFilter> filterList = new LinkedList<>();

    public FilterEditText(Context context) {
        this(context, null);
    }

    public FilterEditText(Context context, AttributeSet attrs) {
        //这里构造方法也很重要，不加这个很多属性不能再XML里面定义
        this(context, attrs, android.R.attr.editTextStyle);
    }

    public FilterEditText(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        init(attrs);
    }

    protected void init(AttributeSet attrs) {
        if (attrs != null) {
            TypedArray typedArray = getContext().obtainStyledAttributes(attrs, R.styleable.FilterEditText);
            maxNum = typedArray.getInt(R.styleable.FilterEditText_maxNum, 0);
        }

        addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                String temp = s.toString().trim();
                //过滤
                for (IJugeFilter jugeFilter : filterList) {
                    if (jugeFilter != null) {
                        temp = filterString(temp, jugeFilter);
                    }
                }

                if (temp.length() != s.length()) {
                    s.clear();
                    s.insert(0, temp);
                }

                //不超过长度限制
                if (maxNum > 0) {
                    if (maxNum < s.length()) {
                        s.delete(maxNum, s.length());
                    }
                }
            }
        });
    }

    public void addFilter(IJugeFilter jugeFilter) {
        filterList.add(jugeFilter);
    }

    interface IJugeFilter {
        Boolean jugeFilter(char oneChar);
    }


    //条件过滤
    public static String filterString(String source, IJugeFilter juge) {
        int len = source.length();
        StringBuilder buf = new StringBuilder(len);
        for (int i = 0; i < len; i++) {
            char codePoint = source.charAt(i);
            if (juge.jugeFilter(codePoint)) {
                buf.append(codePoint);
            }
        }
        return buf.toString();
    }
}
