package com.common.framework.view.customview;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Paint.Align;
import android.graphics.Paint.FontMetrics;
import android.graphics.Rect;
import android.util.AttributeSet;
import android.util.Log;
import android.view.View;

/**
 * Created by chenhongfei on 2015/8/27.
 */
public class HistogramView extends View {
    private int mDensity;
    //  坐标轴 轴线 画笔：
    private Paint axisLinePaint;
    //  坐标轴水平内部 虚线画笔
    private Paint hLinePaint;
    //  绘制文本的画笔
    private Paint titlePaint;
    //  矩形画笔 柱状图的样式信息
    private Paint recPaint;

    // x 轴坐标
    private String[] xTitleArray = new String[]{"0-19", "20-39", "40-59", "60-79", "80-100"};
    // y 轴坐标
    private String[] yTitleArray = new String[]{"100", "80", "60", "40", "20", "0"};

    //x轴的宽度
    private int mXWidth;
    // Y 轴的高度
    private int mYHeight;
    // Y 轴的数值最大值
    private int mMaxValue = 100;
    // Y 轴数值的基本单位
    private int mBaseValue = 1;
    // 柱形图的值
    private int[] thisYear = new int[]{0, 0, 0, 0, 0};
    //
    private int[] lastYear = new int[]{80000, 40000, 20000, 20000, 40000, 80000, 50000};

    // x轴的起点
    private int mBeginX;
    // x轴的终点
    private int mEndX;
    //x轴实际长度
    private int mXAxisLength;
    //x轴上柱面的相对中心点
    private int mXRelativeCenter;
    //x轴数值与x轴的间距
    private int mValueToXSpace;
    //柱面的半宽
    private int mCylinderHalfWidth;
    //数值与柱面之间的间距
    private int mValueToCySpace;


    // 上边距
    private int mMarginTop;
    // y轴的起点
    private int mBeginY;
    // y轴的终点
    private int mEndY;
    //y轴数值与y轴的间距
    private int mValueToYSpace;

    public HistogramView(Context context) {
        super(context);
        // TODO Auto-generated constructor stub
        init(context, null);
    }

    public HistogramView(Context context, AttributeSet attrs) {
        super(context, attrs);
        // TODO Auto-generated constructor stub
        init(context, attrs);
    }

    private void init(Context context, AttributeSet attrs) {
        mDensity = (int) (context.getResources().getDisplayMetrics().density + 0.5f);

        //与x轴相关的值

        mBeginX = 50 * mDensity;

        mXRelativeCenter = 40 * mDensity;
        mValueToXSpace = 14 * mDensity;

        //与柱面相关的值
        mCylinderHalfWidth = 9 * mDensity;
        mValueToCySpace = 5 * mDensity;

        //与y轴相关的值
        mYHeight = 150 * mDensity;
        mBeginY = 5 * mDensity;
        mEndY = mYHeight + mMarginTop;
        mValueToYSpace = 10 * mDensity;
        mMarginTop = 3 * mDensity;

        int textSize = 12 * mDensity;

        axisLinePaint = new Paint();
        hLinePaint = new Paint();
        titlePaint = new Paint();
        titlePaint.setTextSize(textSize);
        recPaint = new Paint();

        axisLinePaint.setColor(Color.DKGRAY);
        hLinePaint.setColor(Color.LTGRAY);
        titlePaint.setColor(0xFF434343);
    }

    private String[] getYTextArray() {
        if (mMaxValue <= 25) {
            return new String[]{"25", "20", "15", "10", "5", "0"};
        } else if (mMaxValue <= 50) {
            return new String[]{"50", "40", "30", "20", "10", "0"};
        } else {
            return new String[]{"100", "80", "60", "40", "20", "0"};
        }
    }

    @Override
    protected void onDraw(Canvas canvas) {
        // TODO Auto-generated method stub
        super.onDraw(canvas);

        mXWidth = getWidth();
        mEndX = mXWidth - 25 * mDensity;
        mXAxisLength = mXWidth - 55 * mDensity;
        // 1 绘制坐标线：
        canvas.drawLine(mBeginX, mBeginY, mBeginX, mEndY, axisLinePaint);
        canvas.drawLine(mBeginX, mEndY, mEndX, mEndY, axisLinePaint);

        // 2 绘制坐标内部的水平线
        //左侧外周的 需要划分的高度：
        String[] yTextArray = getYTextArray();
        int hLineNum = yTextArray.length;// 横线的数量
        int hPerHeight = mYHeight / hLineNum;

        //绘制水平线
        hLinePaint.setTextAlign(Align.CENTER);
        for (int i = 1; i < hLineNum; i++) {
            canvas.drawLine(mBeginX, mMarginTop + i * hPerHeight, mEndX, mMarginTop + i * hPerHeight, hLinePaint);
        }

        // 3 绘制 Y 轴坐标
        FontMetrics metrics = titlePaint.getFontMetrics();
        int descent = (int) metrics.descent;
        titlePaint.setTextAlign(Align.RIGHT);
        for (int i = 0; i < hLineNum; i++) {
            canvas.drawText(yTextArray[i], mBeginX - mValueToYSpace, mMarginTop + (i + 1) * hPerHeight + descent, titlePaint);
        }

        // 4  绘制 X 轴坐标
        titlePaint.setTextAlign(Align.CENTER);
        int columCount = xTitleArray.length + 1;
        int step = mXAxisLength / columCount;
        for (int i = 0; i < columCount - 1; i++) {
            canvas.drawText(xTitleArray[i], mXRelativeCenter + step * (i + 1), mYHeight + mMarginTop + mValueToXSpace, titlePaint);
        }

        // 5 绘制矩形
        if (thisYear != null && thisYear.length > 0) {
            int xCount = xTitleArray.length < thisYear.length ? xTitleArray.length : thisYear.length;
            for (int i = 0; i < xCount; i++) {
                int value = thisYear[i];
                int num = (mMaxValue - value) / mBaseValue;

                // 使用不同颜色的画笔
                switch (i) {
                    case 0:
                        recPaint.setColor(0xFFFF0000);
                        break;
                    case 1:
                        recPaint.setColor(0xFFED8506);
                        break;
                    case 2:
                        recPaint.setColor(0xFFEEE408);
                        break;
                    case 3:
                        recPaint.setColor(0xFF3FE406);
                        break;
                    case 4:
                        recPaint.setColor(0xFF168E04);
                        break;
                    default:
                        recPaint.setColor(0xFFFF0000);
                        break;
                }

                Rect rect = new Rect();
                rect.left = mXRelativeCenter + step * (i + 1) - mCylinderHalfWidth;
                rect.right = mXRelativeCenter + step * (i + 1) + mCylinderHalfWidth;

                //当前的相对高度：
                int rh = (mYHeight - hPerHeight) * num / (mMaxValue / mBaseValue) + hPerHeight;

                rect.top = rh + mMarginTop;
                rect.bottom = mEndY;
                Log.v("HistogramView", String.format("%d, %d, %d, %d, %d, %d", mYHeight, num, mMaxValue, mBaseValue, rect.top, rect.bottom));

                canvas.drawRect(rect, recPaint);
                canvas.drawText(Integer.toString(value), rect.left + mCylinderHalfWidth, rect.top - mValueToCySpace, titlePaint);
            }
        }

//        if (lastYear != null && lastYear.length > 0) {
//            int thisCount = lastYear.length;
//            for (int i = 0; i < thisCount; i++) {
//                int value = lastYear[i];
//                int num = 8 - value / 10000;
//
//                recPaint.setColor(0xFF0000FF);
//
//                Rect rect = new Rect();
//
//                rect.left = 100 + step * (i + 1) - 10;
//                rect.right = 100 + step * (i + 1) + 10;
//
////              当前的相对高度：
//                int rh = (leftHeight * num) / 8;
//
//                rect.top = rh + 20;
//                rect.bottom = 320;
//
//                canvas.drawRect(rect, recPaint);
//            }
//        }
    }

    /**
     * 跟新自身的数据 需要View子类重绘。
     * <p/>
     * 主线程 刷新控件的时候调用：
     * this.invalidate();  失效的意思。
     * this.postInvalidate();  可以子线程 更新视图的方法调用。
     */
    //updata this year data
    public void updateThisData(int[] thisData) {
        thisYear = thisData;
//      this.invalidate(); //失效的意思。
        this.postInvalidate();  //可以子线程 更新视图的方法调用。
    }

    //updata last year data
    public void updateLastData(int[] lastData) {
        lastYear = lastData;
//      this.invalidate(); //失效的意思。
        this.postInvalidate();  //可以子线程 更新视图的方法调用。
    }

    public void setSimpleData(int[] data, int maxValue) {
        this.thisYear = data;
        if (maxValue <= 25) {
            this.mMaxValue = 25;
        } else if (maxValue <= 50) {
            this.mMaxValue = 50;
        } else {
            this.mMaxValue = 100;
        }
        this.postInvalidate();
    }
}
